import unittest
import pybald
from pybald import context
from six import StringIO
from pybald.config import constants

test_constants = StringIO('''
[copyright]
year: 2017

[marketing]
burgers_served: billions and billions

[other]
phone: 555 555 1234
''')


class TestConstants(unittest.TestCase):
    def setUp(self):
        # rewind the constants file each time
        test_constants.seek(0)
        constants_values = constants.read(constants_file=test_constants)
        context = pybald.configure(page_options={'constants': constants_values})

    def tearDown(self):
        context._reset()

    def test_constant_copyright(self):
        self.assertEqual(context.render.raw_template(b'${constants.copyright.year}'), b'2017')

    def test_constant_marketing(self):
        print(context.config)
        self.assertEqual(context.render.raw_template(b'${constants.marketing.burgers_served}'), b'billions and billions')

    def test_constant_other(self):
        self.assertEqual(context.render.raw_template(b'${constants.other.phone}'), b'555 555 1234')



from six import with_metaclass
import sqlalchemy
import sqlalchemy.orm

import sqlalchemy.ext.declarative
from sqlalchemy.ext.declarative import declarative_base

from sqlalchemy.orm.attributes import (
    instance_state
    )
from sqlalchemy.orm.util import (
    has_identity
)
from sqlalchemy.dialects import postgresql as pg

from pybald import context
from pybald.util import camel_to_underscore, pluralize

from pybald.db import ext

# the surrogate_pk template that assures that surrogate primary keys
# are all the same and ordered with the pk first in the table
surrogate_pk_template = sqlalchemy.Column(sqlalchemy.Integer, nullable=False,
                                          primary_key=True)


def _include_sqlalchemy(obj):
    for module in sqlalchemy, sqlalchemy.orm, ext:
        for key in module.__all__:
            if not hasattr(obj, key):
                setattr(obj, key, getattr(module, key))


def make_model_class(Base, session, config=None):
    '''Create a series of classes for models that are bound to a context.'''
    if config is None:
        class BlankConfig(object):
            def __getattr__(self, key):
                return None

        config = BlankConfig()

    class ModelMeta(sqlalchemy.ext.declarative.DeclarativeMeta):
        '''
        MetaClass that sets up some common behaviors for pybald models.

        Will assign tablename if not defined based on pluralization rules. Also
        applies project global table arguments. Lastly this will assign a
        numeric primary key id to the table if no primary key was defined
        in the model class.
        '''
        def __init__(cls, name, bases, ns):
            try:
                if Model not in bases:
                    # Allow for inheritance of the Model object
                    # for use with SqlAlchemy inheritance patterns
                    super(ModelMeta, cls).__init__(name, bases, ns)
                    return
            except NameError:
                # if this is the Model class, then just return
                return

            # set the tablename, if it's user set, use that, otherwise use a
            # function to create one
            cls.__tablename__ = getattr(cls, "__tablename__",
                                        pluralize(camel_to_underscore(name)))
            # tableargs adds autoload to create schema reflection
            cls.__table_args__ = getattr(cls, "__table_args__", {})
            if config.schema_reflection:
                # create or update the __table_args__ attribute
                cls.__table_args__['autoload'] = True
            if config.global_table_args:
                cls.__table_args__.update(config.global_table_args)

            auto_create_pk = getattr(cls, "__auto_create_pk__", True)

            if cls.__table_args__.get('autoload', False):
                auto_create_pk = False

            # check if we're auto creating PKs and if the class has at least
            # one primary key if not, automatically generate an integer
            # surrogate primary key.
            if auto_create_pk:
                has_primary = any([False] + [value.primary_key
                                             for value in cls.__dict__.values()
                                             if isinstance(value, sqlalchemy.Column)])
                if not has_primary:
                    # treat the id as a mixin w/copy
                    surrogate_pk = surrogate_pk_template.copy()
                    surrogate_pk._creation_order = surrogate_pk_template._creation_order
                    cls.id = surrogate_pk
            super(ModelMeta, cls).__init__(name, bases, ns)

    class RegistryMount(type):
        '''
        A registry creating metaclass that keeps track of all defined classes that
        inherit from a base class using this metaclass.
        '''
        # lifted almost verbatim from: http://martyalchin.com/2008/jan/10/simple-plugin-framework/
        def __init__(cls, name, bases, attrs):
            try:
                cls.registry.append(cls)
            except AttributeError:
                # this is processing the first class (the mount point)
                cls.registry = [] #context.model_registry

            return super(RegistryMount, cls).__init__(name, bases, attrs)

    class RegistryModelMeta(RegistryMount, ModelMeta):
        pass

    class Model(with_metaclass(RegistryModelMeta, Base)):
        '''Pybald Model class, inherits from SQLAlchemy Declarative Base.'''
        NotFound = sqlalchemy.orm.exc.NoResultFound
        MultipleFound = sqlalchemy.orm.exc.MultipleResultsFound

        def is_modified(self):
            '''Check if SQLAlchemy believes this instance is modified.'''
            return instance_state(self).modified

        def clear_modified(self):
            '''
            Unconditionally clear all modified state from the attibutes on
            this instance.
            '''
            return instance_state(self)._commit_all({})

        def is_persisted(self):
            '''
            Check if this instance has ever been persisted. Means it is either
            `detached` or `persistent`.
            '''
            return has_identity(self)

        def save(self, flush=False):
            '''
            Saves (adds) this instance in the current databse session.

            This does not immediatly persist the data since these operations
            occur within a transaction and the sqlalchemy unit-of-work pattern.
            If something causes a rollback before the session is committed,
            these changes will be lost.

            When flush is `True`, flushes the data to the database immediately
            **within the same transaction**. This does not commit the transaction,
            for that use :py:meth:`~pybald.db.models.Model.commit`)
            '''

            session.add(self)

            if flush:
                self.flush()
            return self

        def delete(self, flush=False):
            '''
            Delete this instance from the current database session.

            The object will be deleted from the database at the next commit. If
            you want to immediately delete the object, you should follow this
            operation with a commit to emit the SQL to delete the item from
            the database and commit the transaction.
            '''
            session.delete(self)
            if flush:
                self.flush()
            return self

        def flush(self):
            '''
            Syncs all pending SQL changes (including other pending objects) to
            the underlying data store within the current transaction.

            Flush emits all the relevant SQL to the underlying store, but does
            **not** commit the current transaction or close the current
            database session.
            '''
            session.flush()
            return self

        def commit(self):
            '''
            Commits the entire database session (including other pending objects).

            This emits all relevant SQL to the databse, commits the current
            transaction, and closes the current session (and database connection)
            and returns it to the connection pool. Any data operations after this
            will pull a new database session from the connection pool.
            '''
            session.commit()
            return self

        @classmethod
        def get(cls, **where):
            '''
            A convenience method that constructs a load query with keyword
            arguments as the filter arguments and return a single instance.
            '''
            return cls.load(**where).one()

        @classmethod
        def all(cls, **where):
            '''
            Returns a collection of objects that can be filtered for
            specific collections.

            all() without arguments returns all the items of the model type.
            '''
            return cls.load(**where).all()

        # methods that return queries (must execute to retrieve)
        # =============================================================
        @classmethod
        def load(cls, **where):
            '''
            Convenience method to build a sqlalchemy query to return stored
            objects.

            Returns a query object. This query object must be executed to retrieve
            actual items from the database.
            '''
            if where:
                return session.query(cls).filter_by(**where)
            else:
                return session.query(cls)

        @classmethod
        def filter(cls, *pargs, **kargs):
            '''
            Convenience method that auto-builds the query and passes the filter
            to it.

            Returns a query object.
            '''
            return session.query(cls).filter(*pargs, **kargs)

        @classmethod
        def query(cls):
            '''
            Convenience method to return a query based on the current object
            class.
            '''
            return session.query(cls)

        @classmethod
        def show_create_table(cls):
            '''
            Uses the simple dump_engine to print to stdout the SQL for this
            table.
            '''
            cls.__table__.create(context.dump_engine)

    class NonDbModel(with_metaclass(RegistryMount, object)):
        '''
        A plain Python object that uses the RegistryMount system to register
        non database models. Inheriting from this allows the object to be registered
        as a Model.
        '''
        # __metaclass__ = RegistryMount
        # use the same registry space for NonDbModels
        registry = Model.registry

    return Model, NonDbModel


class ContextBoundModels(object):
    '''Creates a 'bound' set of model definitions to a particular context'''
    def __init__(self, config=None):
        if config:
            self.config = config
        else:
            self.config = context.config
        _include_sqlalchemy(self)
        self.engine = self.create_engine(self.config.database_engine_uri,
                                         **self.config.database_engine_args)
        self.db = self.create_session(engine=self.engine)
        self.Base = declarative_base(bind=self.engine)
        self.Model, self.NonDbModel = make_model_class(self.Base, self.db, self.config)

    def create_session(self, engine=None, session_args=None):
        # build session
        if session_args is None:
            session_args = {}
        session = self.scoped_session(self.sessionmaker(bind=engine, **session_args))
        return session



import unittest
import shutil
import tempfile
import os
from webob import Request
import pybald
from pybald import context
from pybald.util.static_serve import StaticServer

import logging
log = logging.getLogger(__name__)

# binary data for a tiny png file
binary_file_data = b'''\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00!\x00\x00\x00\x1a\x08\x03\x00\x00\x00\x0e\x87\xc53\x00\x00\x00\x04gAMA\x00\x00\xb1\x8f\x0b\xfca\x05\x00\x00\x00\x01sRGB\x00\xae\xce\x1c\xe9\x00\x00\x01\xecPLTEGpL\xbc\xbc\xbb\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xe3\xe3\xe3\xff\xff\xff\x00\x00\x00]]]\x00\x00\x00\xc2\xc2\xc1\xfe\xff\xff\xfb\xfb\xfb\x96\x96\x96\xff\xff\xff\xba\xba\xba\xeb\xea\xeb\x14\x14\x14\xbe\xbe\xbe\x1d\x1d\x1d\xe0\xe0\xe0\xf8\xf8\xf8\xf8\xf8\xf7\xba\xc7\xae\xee\xee\xee\xf3\xf4\xf2\xd9\xd9\xd9\xc0\xc0\xc0\xae\xae\xae\xe1\xe1\xe1\xb9\xb9\xb7\xf1\xf1\xf1\xc9\xd3\xc0\xe7\xe7\xe7\xd3\xdb\xcd\xfe\xfe\xfe\xc4\xcf\xb9\xd2\xd2\xd2\xd7\xd7\xd7\xd3\xd3\xd3\xf4\xf4\xf4```\x85\x85\x85\xe2\xe2\xe2\xdb\xdb\xdb\xf6\xf6\xf6\xda\xda\xda\xeb\xeb\xeb\xf7\xf7\xf7\xf4\xf4\xf4\xef\xef\xef\xd7\xd7\xd7\xfe\xfe\xfe\xfa\xfa\xfaDDD\xed\xed\xec\x00\x00\x00\xec\xec\xec\x00\x00\x00\xec\xec\xec\xbf\xbf\xbf\xe1\xe7\xdc\xa5\xb8\x95\xe8\xed\xe5\xce\xce\xce\xec\xec\xec\xeb\xeb\xeb\xe9\xe9\xe9\xe7\xe7\xe7\xe4\xe4\xe4\xe3\xe3\xe3\xec\xe6\xe7\xf5\xf5\xf5\xdb\xdb\xdb\xe5\xda\xdb\xd6\xd6\xd6\xef\xee\xee\xd8\xd8\xd8\xac\xac\xac\xcd\xcd\xcd\xaa\xaa\xa4\xb4\xb4\xb4\xe2\xe2\xe2\xe5\xe7\xe7\xde\xde\xde\xcc\xcc\xcc\xec\xec\xec\xcb\xcb\xcb\xc0\xc0\xc0\xd6\xd6\xd6\xd7\xd7\xd7\x89\x8e\x8e\xd8\xd8\xd8\xa9\xa9\xa5\xe2\xe2\xe2\xc0\xc0\xc0\xdc\xdc\xdc\xe6\xe6\xe6\xf2\xf2\xf2hhh\x00\x00\x00\xf0\xf0\xf0\xb2\xb2\xb2\xb4\xb4\xb4\xae\xbe\xa1\x8b\xa5ru\x97S\xff\xff\xff\xfe\xfe\xfe\xef\xef\xf0\xf1\xf1\xf1\xfc\xfc\xfc\xed\xed\xed\xf5\xf5\xf5\xf2\xf3\xf3\xfd\xfe\xfd\xf4\xf4\xf4\xfa\xfa\xf9\xfa\xfb\xf9\xf8\xf8\xf8\xb1\xc1\xa3\xdf\xdf\xdfz\x9aZ\x98\xae\x85\xe9\xe9\xe9\xa9\xbb\x99\xd6\xde\xcf\xd9\xe0\xd3\xed\xf0\xe9\xf6\xf8\xf4\x83\x9fg\xce\xd8\xc6\xee\xef\xef\xef\xf2\xec\x92\xab|\xc1\xcd\xb6\xc6\xd1\xbc\xdc\xe3\xd6\xde\xe5\xd9\xe2\xe2\xe2\xe4\xe5\xe4\xbb\xc9\xb0\xe9\xee\xe5r\x96Lw\x98R\xa0\xb4\x8di\x90;\x8c\xa5u\x87\xa2m\xb6\xc5\xa8\xe2\xd1\xd3\xd6\xb6\xb9\xd7\xd7\xd7\xcd\xcd\xcdo\x94C\xb6\xc2\xac\x7f\x9dc\x8e\xa7ua\x8b*(s\x00\x92\xa9\x7fj\x8f?S_`q\x00\x00\x00\x8etRNS\x00\xba\xfb\xfc\xfd\x03\xfc\x02\xfe\x0e\'\x199\xfe\xfe\x01\xfa\xba\xe0\x10\xba\x04\x80\xf7\xfd\xfe\xeb\xfce4#y/\xe9\xfc\xfe\xfe\xfc\xfeTbY\xe2\x1b\x1a\xec\x07\xfbq\xc1\xef\xf0\xe1\xc2\xf8\xef\x1c\xfa\n\xdd\x08\xea\xc1\xfe\xfe\xfcI\xc9\xc9\xaf\xa0\xad\x93\xfa\xf6\xa1\xfb}\xd3\xe6(Q*\\\xf0\x95u}\xb3\x89\x8a{Z4\xb1G\xdf\xfe\x9b\xd8\xf8 \x15\xee\xa6\x99\xfc\xfc\xfc\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xff\xfe\xff\xfe\xcb\x14\xdbZ\x00\x00\x02VIDAT(\xcfu\x90\x07S"A\x10\x85[jY\xe6\x14\x84\x13I\x87Y\x8ce,\x15\xc3\x19/\xe7\x9cs\xce\xbd\x01\t.Q\xd2*`\x8e\x97\xe3\x1f\xbda\x17,\xbc+_\xd5\x84\xea\xfe\xe6Mw\x03\xec\x93N\x0bPW\x7f\xa7\xff\xae\x19\x8c\xfb3`\xd4*\x02\x18\xb9\xd7\xcdq>\xbc\r=\xc6\x7f\x9f*\xea\xbax\x191\xe0v\xe3}\x1a\xd4\x95\x00\x00\x96\x86cm\x1d\xedg:9QS\xc9{\x03\xdeG\x0f\xba\x94x\x11\x18\xaf9n\xf3a^\xce^\x94B\x9f\xbf,#6Z\xc7\x8b\x88\x0ejO(Y\xc2\xb0\xac\xb4,\x13z\xf3\xf72\x88\xa7-j\xb9Z\xa8kD\x8d\x89!\x84\x88\x12\xef\xf1\xf0Q\xa7f\xb4\xf9)ob\xb0\r\xb4*aEw\x0b\xad\xcf+{d\x7f%1\xb5\xc8\x18\x8dF\x9bE\x16\xeb\x15\x82\xfa\x9cD\x96COH\x08\xe4\x7f\xa2kv\xe8\xd90\xca\x02\xc1\x1a\xd5\xc3\x08\xa7\x90%(\t\x82,\x0b~\xe9\\\x7f_\x1d\xc0\xc3\x1b\x9f.\xa0]\xaf\x96\xaa\x85vd8D\xf4UJ~A\xe8>\xdfWSo\xbdi\xfbx\x16/\x8d\xa9\x84\x0e\xcc\x83\xe8F\x0e\xd5~\xb0 \x83\t\xad\x85n\xb5=\xd0\x81\xc8\x10N\x11\xcd1\x0c\xed\x9a\xd3\xe0\xc0\x15\xe8\xd9\x1b\xf6\xd5\xce\xbd\xd7\x9c\xc1` \x84#\x0c\xde*$-\rz\xbd\xfe\xda\xf5\xa1Q\xbf$\x8a>_\xf1\x13\x16\x9f<\xae\xd5\xd7\xea\x1b,\xf0\xea9\xda\xec\xf6\xe1\x01\'\x1fj\x96\xa9x\xaf$\x06$:\x9b\x174l\xe3\xaa^B\xc5`\xd1_\xa2\xf3\xe4\xa3+\xa1\x15z\xce\x06\nn\xaf\xcb\xa1\xe2(\xba\r\x0c\x93\xa78\x8c\x7fS\xe3s\xab\t$\xad\xf3\x8c\x1b\'(Q\x85-\xb80\x15_X[\x10\x13\x99\xef\xf1\xc4\\\xdc\x95L\xfe\\\x9as}]\xa5\x99#\x05"\xb8\xb1\x18\xd9\x0c\xce\xff\xc8\x86#\xe1\\&\xbc\xbd\xb6\xed\x8a,\xae\xb7\x96\x101Gr#\xbb\x18N\xbb"\x98\nfc\xe1\\$\x98\xf2M;J=~\xc7R\x81\xcd_\xc9\xf4\xd6\xeeN\xecOl=\xbe\x93I\x05\xb7\x8a\x04\xad\x94}\xf7!3\xc3N\xbfg\x13M\x8e){\xb6\xc9\xe5t9s\x8e\xa6\x19\xd6\x84\x8d\x94\x98@\xc4t\x9an\xbbK\xf8\x9f\xc8h9\x98\xdfN\x96U\x97\x1d\xce\xabZ9\xd5\xa5D\xaa\xcb&\xdf\x8c\x00\x8c\x9b\x0f\x1d,\xf3\x98\xf1/\xdd\xa1\xd3/;\\G\x8c\x00\x00\x00\x00IEND\xaeB`\x82'''

test_conf = dict(env_name="SampleTestProjectEnvironment",
                 debug=False)


class TestStaticServer(unittest.TestCase):
    def setUp(self):
        context._reset()
        self.test_dir = tempfile.mkdtemp()
        with open(os.path.join(self.test_dir, "pybald.png"), "wb") as image:
            image.write(binary_file_data)

    def tearDown(self):
        # Remove the directory after the test
        shutil.rmtree(self.test_dir)
        context._reset()

    def test_serve_png_file(self):
        "Serve a file from a static asset path (temporary)"
        pybald.configure(config_object=test_conf)
        app = StaticServer(application=None, path=self.test_dir)

        resp = Request.blank('/pybald.png').get_response(app)
        self.assertEqual(resp.headers['Content-Type'], 'image/png')
        self.assertEqual(resp.headers['Content-Length'], '1342')
        self.assertEqual(resp.body, binary_file_data)



if __name__ == "__main__":
    unittest.main()



'''Copyright (c) Django Software Foundation and individual contributors.
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without modification,
	are permitted provided that the following conditions are met:
	
	    1. Redistributions of source code must retain the above copyright notice,
	       this list of conditions and the following disclaimer.
	   
	    2. Redistributions in binary form must reproduce the above copyright
	       notice, this list of conditions and the following disclaimer in the
	       documentation and/or other materials provided with the distribution.
	
	    3. Neither the name of Django nor the names of its contributors may be used
	       to endorse or promote products derived from this software without
	       specific prior written permission.
	
	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
	ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
	WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
	DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
	ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
	(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
	LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
	ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
	SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
from math import ceil

class InvalidPage(Exception):
    pass

class PageNotAnInteger(InvalidPage):
    pass

class EmptyPage(InvalidPage):
    pass

class Paginator(object):
    def __init__(self, object_list, per_page, orphans=0, allow_empty_first_page=True):
        self.object_list = object_list
        self.per_page = per_page
        self.orphans = orphans
        self.allow_empty_first_page = allow_empty_first_page
        self._num_pages = self._count = None

    def validate_number(self, number):
        "Validates the given 1-based page number."
        try:
            number = int(number)
        except ValueError:
            raise PageNotAnInteger('That page number is not an integer')
        if number < 1:
            raise EmptyPage('That page number is less than 1')
        if number > self.num_pages:
            if number == 1 and self.allow_empty_first_page:
                pass
            else:
                raise EmptyPage('That page contains no results')
        return number

    def page(self, number):
        "Returns a Page object for the given 1-based page number."
        number = self.validate_number(number)
        bottom = (number - 1) * self.per_page
        top = bottom + self.per_page
        if top + self.orphans >= self.count:
            top = self.count
        return Page(self.object_list[bottom:top], number, self)

    def _get_count(self):
        "Returns the total number of objects, across all pages."
        if self._count is None:
            try:
                self._count = self.object_list.count()
            except (AttributeError, TypeError):
                # AttributeError if object_list has no count() method.
                # TypeError if object_list.count() requires arguments
                # (i.e. is of type list).
                self._count = len(self.object_list)
        return self._count
    count = property(_get_count)

    def _get_num_pages(self):
        "Returns the total number of pages."
        if self._num_pages is None:
            if self.count == 0 and not self.allow_empty_first_page:
                self._num_pages = 0
            else:
                hits = max(1, self.count - self.orphans)
                self._num_pages = int(ceil(hits / float(self.per_page)))
        return self._num_pages
    num_pages = property(_get_num_pages)

    def _get_page_range(self):
        """
        Returns a 1-based range of pages for iterating through within
        a template for loop.
        """
        return range(1, self.num_pages + 1)
    page_range = property(_get_page_range)

QuerySetPaginator = Paginator # For backwards-compatibility.

class Page(object):
    def __init__(self, object_list, number, paginator):
        self.object_list = object_list
        self.number = number
        self.paginator = paginator

    def __repr__(self):
        return '<Page %s of %s>' % (self.number, self.paginator.num_pages)

    def has_next(self):
        return self.number < self.paginator.num_pages

    def has_previous(self):
        return self.number > 1

    def has_other_pages(self):
        return self.has_previous() or self.has_next()

    def next_page_number(self):
        return self.number + 1

    def previous_page_number(self):
        return self.number - 1

    def start_index(self):
        """
        Returns the 1-based index of the first object on this page,
        relative to total objects in the paginator.
        """
        # Special case, return zero if no items.
        if self.paginator.count == 0:
            return 0
        return (self.paginator.per_page * (self.number - 1)) + 1

    def end_index(self):
        """
        Returns the 1-based index of the last object on this page,
        relative to total objects found (hits).
        """
        # Special case for the last page because there can be orphans.
        if self.number == self.paginator.num_pages:
            return self.paginator.count
        return self.number * self.paginator.per_page



import unittest

class TestEngine(unittest.TestCase):
    def setUp(self):
        pass

    # def test_nothing(self):
    @unittest.skip("unconfigured exceptions not working in Py3")
    def test_unconfigured_engine(self):
        "Raise an exception when the db engine is not configured"
        with self.assertRaises(RuntimeError) as context:
            from pybald.context import models
            models.engine.execute("SELECT 1234")
        # except RuntimeError as err:
        #     pass
        # else:
        #     self.fail('Runtime exception not thrown')


import json
import pickle
import zlib

# from sqlalchemy.orm.interfaces import SessionExtension
from sqlalchemy.ext.mutable import Mutable

from sqlalchemy.types import (
    TypeDecorator,
    VARCHAR,
    PickleType
    )

# py3 compatibility
try:
    unicode
except NameError:
    unicode = str
from six import iteritems


class ASCII(TypeDecorator):
    '''
    A database string type that only allows ASCII characters.

    This is a data type check since all strings in the database could
    be unicode.
    '''
    impl = VARCHAR

    def process_bind_param(self, value, dialect):
        '''
        Run encode on a unicode string to make sure the string only
        contains ASCII characterset characters. To avoid another conversion
        pass, the original unicode value is passed to the underlying db.
        '''
        # run an encode on the value with ascii to see if it contains
        # non ascii. Ignore the return value because we only want to throw
        # an exception if the encode fails. The data can stay unicode for
        # insertion into the db.
        if value is not None:
            value.encode('ascii')
        return value

    def process_result_value(self, value, dialect):
        '''
        Run encode on a unicode string coming out of the database to turn the
        unicode string back into an encoded python bytestring.
        '''
        if isinstance(value, unicode):
            value = value.encode('ascii')
        return value


class JSONEncodedDict(TypeDecorator):
    '''
    Represents a dictionary as a json-encoded string in the database.
    '''
    impl = VARCHAR

    def process_bind_param(self, value, dialect):
        '''
        Turn a dictionary into a JSON encoded string on the way into
        the database.
        '''
        if value is not None:
            value = json.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        '''
        Turn a JSON encoded string into a dictionary on the way out
        of the database.
        '''
        if value is not None:
            value = json.loads(value)
        return value


class ZipPickler(object):
    '''Simple wrapper for pickle that auto compresses/decompresses values'''
    def loads(self, string):
        return pickle.loads(zlib.decompress(string))

    def dumps(self, obj, protocol):
        return zlib.compress(pickle.dumps(obj, protocol))


class ZipPickleType(PickleType):
    def __init__(self, *pargs, **kargs):
        super(ZipPickleType, self).__init__(pickler=ZipPickler(), *pargs, **kargs)


class MutationDict(Mutable, dict):
    '''
    A dictionary that automatically emits change events for SQA
    change tracking.

    Lifted almost verbatim from the SQA docs.
    '''
    @classmethod
    def coerce(cls, key, value):
        "Convert plain dictionaries to MutationDict."
        if not isinstance(value, MutationDict):
            if isinstance(value, dict):
                return MutationDict(value)
            # this call will raise ValueError
            return Mutable.coerce(key, value)
        else:
            return value

    def update(self, *args, **kwargs):
        '''
        Updates the current dictionary with kargs or a passed in dict.
        Calls the internal setitem for the update method to maintain
        mutation tracking.
        '''
        for k, v in iteritems(dict(*args, **kwargs)):
            self[k] = v

    def __setitem__(self, key, value):
        '''Detect dictionary set events and emit change events.'''
        dict.__setitem__(self, key, value)
        self.changed()

    def __delitem__(self, key):
        '''Detect dictionary del events and emit change events.'''
        dict.__delitem__(self, key)
        self.changed()

    def __getstate__(self):
        '''Get state returns a plain dictionary for pickling purposes.'''
        return dict(self)

    def __setstate__(self, state):
        '''
        Set state assumes a plain dictionary and then re-constitutes a
        Mutable dict.
        '''
        self.update(state)

    def pop(self, *pargs, **kargs):
        """
        Wrap standard pop() to trigger self.changed()
        """
        result = super(MutationDict, self).pop(*pargs, **kargs)
        self.changed()
        return result

    def popitem(self, *pargs, **kargs):
        """
        Wrap standard popitem() to trigger self.changed()
        """
        result = super(MutationDict, self).popitem(*pargs, **kargs)
        self.changed()
        return result

__all__ = ['ASCII', 'JSONEncodedDict', 'ZipPickler', 'MutationDict', 'ZipPickleType']



import unittest
import pybald
from pybald.core.controllers import Controller
from pybald.db import models
from pybald import context

class TestRegistries(unittest.TestCase):
    def setUp(self):
        config_obj = dict(env_name="TestFromObject",
                          database_engine_uri='sqlite:///:memory:')
        pybald.configure(config_object=config_obj)

    def tearDown(self):
        context._reset()

    def test_controller_registry(self):
        "Create a controller, it appears in the Controller registry"
        class AnotherSampleController(Controller):
            pass
        assert AnotherSampleController in Controller.registry

    def test_model_registry(self):
        "Create a model, it appears in the Model registry"
        class AnotherSampleModel(models.Model):
            pass
        assert AnotherSampleModel in models.Model.registry


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# encoding: utf-8
# """
# Console.py
#
# Created by mikepk on 2014-10-16.
# Copyright (c) 2015 Michael Kowalchik. All rights reserved.
# """
import code

import readline
import atexit
import os

import webob
from ..test import Client


class Console(code.InteractiveConsole):
    '''Pybald console including history buffer per project.'''
    def __init__(self, project_name, package_name=None, app=None,
                 additional_symbols=None):
        if additional_symbols is None:
            additional_symbols = {}

        self.project_name = project_name
        if not package_name:
            package_name = self.project_name

        # add the application to the console namespace
        console_symbols = {'app': app}

        # add webob Req/Resp objects to the console for convenience
        console_symbols.update({"Request": webob.Request,
                                "Response": webob.Response})
        console_symbols.update({"c": Client(app=app)})
        console_symbols.update(additional_symbols)

        histfile = os.path.expanduser("~/.pybald-{0}-console-history".format(
                                                                 project_name))

        code.InteractiveConsole.__init__(self, locals=console_symbols,
                                                          filename="<console>")
        self.init_history(histfile)

    def init_history(self, histfile):
        readline.parse_and_bind("tab: complete")
        if hasattr(readline, "read_history_file"):
            try:
                readline.read_history_file(histfile)
            except IOError:
                pass
            atexit.register(self.save_history, histfile)

    def save_history(self, histfile):
        readline.write_history_file(histfile)

    def run(self):
        '''Start the console with the project, controllers,
        and models defined in the console's namespace.'''
        self.interact('''Welcome to the Pybald interactive console\n'''
                      ''' ** project: {0} **'''.format(self.project_name))


def start_console(app, options=None):
    '''Start a console with a particular app.

    :param app: wsgi application passed in
    :param options: an object or named tuple containing the options for the
                    web server such as host and port. Generally an argparser
                    object is passed in for command line invokation

    A pybald application must be configured before starting a console.
    '''
    import pybald
    from pybald.db import models
    # now the models registry is loaded and the additional_symbols
    # added so models are available in the console
    # if no models are configured, return empty symbols
    try:
        symbols = dict([(model.__name__, model) for model in
                        models.Model.registry])
    except RuntimeError:
        symbols = {}
    else:
        symbols['models'] = models
        symbols['db'] = pybald.context.db
    # create a pybald console around it
    console = Console(project_name=pybald.context.config.project_name or
                      pybald.context.name, app=app,
                      additional_symbols=symbols)
    console.run()



'''The constants helper module is to allow for creating 'safer' ini files for
values that will be injected into templates. The basic rationale is that there
are often values that you want non-technical people to control (phone numbers,
marketing claims, addresses) and similar values. Using constants allows these
individuals to use friendlier ini format but still have the values available
for page templates.
'''
# TODO: The constants helper object should probably do something 'safe' when
# constants are deleted, maybe issuing a warning. Just a saftey net to avoid
# breaking templates if sections or values are accidently deleted.
try:
    from ConfigParser import ConfigParser as configparser
except ImportError:
    from configparser import ConfigParser as configparser
from collections import namedtuple

# generate constants for templates
# this is a little squirrley looking, but all it does is use the config
# parser to read an INI file and then generates a two level set of named
# tuples to surface the data. Inside the template the section/data of the
# constants ini is dereferenced using constants.SECTION.KEY


def read(filename='constants.ini', constants_file=None):
    '''Read a constants ini file and return the constants helper object.

    When added to the page_options config, these constants can be used inside
    the templates where the section/data of the constants ini is dereferenced
    using constants.SECTION.KEY.
    '''
    config = configparser()
    if constants_file:
        config.readfp(constants_file)
    else:
        config.read(filename)
    constants = namedtuple("Constant", config.sections())._make([
        namedtuple('Field', dict(config.items(section)).keys()
            )._make(dict(config.items(section)).values())
                for section in config.sections() ])
    return constants



#!/usr/bin/env python
# encoding: utf-8

from webob import Response, exc
# from six.moves.urllib.parse import unquote
import logging
log = logging.getLogger(__name__)


def create_default_error_controller():
    from pybald.core.controllers import Controller, action
    from pybald.context import render, config
    class DefaultErrorController(Controller):
        '''Minimum helpful error controller'''
        def __init__(self, exception=None, status_code=500, message=None):
            self.status_code = status_code
            self.message = message
            self.exception = exception

        @action
        def __call__(self, req):
            '''
            The ErrorController will return a formatted stack trace if in debug
            mode, or point to a regular error page otherwise.
            '''
            if 400 <= int(self.status_code) < 500:
                return req.get_response(self.exception)
            else:
                if config.debug:
                    stack_trace = render(template='stack_trace', data={'req': req})
                    return Response(body=stack_trace, status=self.status_code)
                else:
                    return req.get_response(exc.HTTPServerError('General Fault'))
    return DefaultErrorController


class ErrorMiddleware(object):
    '''
    Handles exceptions during web transactions.

    Implemented as WSGI middleware, this middleware wraps the application and
    intercepts errors and exceptions. The middleware makes a distinction between
    webob HTTP exceptions and other exception types.


    :param application:  WSGI application/middleware that is to be
                         *wrapped* by the error handler in the web app pipeline.

    :param error_controller:  A class that is used by the middleware to handle
                          errors for the project. When configured, an error_controller
                          instance is created and the WSGI response is passed to
                          the error_controller to handle.
    '''
    def __init__(self, application=None, error_controller=None):
        if error_controller is None:
            self.error_controller = create_default_error_controller()
        else:
            self.error_controller = error_controller

        if application:
            self.application = application
        else:
            # no pipeline so just generate a generic response
            self.application = Response()

    def __call__(self, environ, start_response):
        #pass through if no exceptions occur
        try:
            return self.application(environ, start_response)
        # handle HTTP errors
        except exc.HTTPException as err:
            log.debug(u"{0} Thrown: {1}".format(err.__class__.__name__, err))
            # if the middleware is configured with an error controller
            # use that to display the errors
            if self.error_controller:
                handler = self.error_controller(err, status_code=err.code)

                try:
                    # try executing error_handler code
                    # otherwise re-raise the exception
                    return handler(environ, start_response)
                except Exception:
                    log.exception("Exception thrown during error_controller handling")
                    raise
            else:
                # HTTPExceptions are also WSGI apps and can be called as such
                return err(environ, start_response)
        except Exception as err:
            log.exception("General Exception thrown")
            if self.error_controller:
                handler = self.error_controller(err, message=str(err))
            else:
                # create a generic HTTP server Error webob exception
                handler = exc.HTTPServerError('General Fault')
            return handler(environ, start_response)



'''
Test Module for pybald including a simple client for issuing web-like
requests against the WSGI application pipeline.
'''
from webob import Request
from six.moves.urllib.parse import urlencode

import re


class Client(object):
    '''Web testing client for pybald'''
    def __init__(self, app):
        self.app = app
        self.cookies = {}

    def _set_cookie(self, set_cookie):
        '''Naively sets cookies for this test client session'''
        match = re.search(r'([^\;]*)\;.*', set_cookie)
        if match:
            # cookie = match.group(1)
            key, value = match.group(1).split('=')
            self.cookies[key] = value

    def get(self, url):
        '''Issue a GET request directly against the WSGI pybald app'''
        req = Request.blank(url)
        for key, value in self.cookies.items():
            req.cookies[key] = value
        resp = req.get_response(self.app)
        # naively handle cookies. This can be made more robust later
        cookie_string = resp.headers.get("set-cookie")
        if cookie_string:
            self._set_cookie(cookie_string)
        return resp

    def clear_cookies(self):
        '''Delete all cookies from this session.'''
        self.cookies = {}

    def post(self, url, data):
        '''Issue a POST request directly against the WSGI pybald app'''
        req = Request.blank(url,
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body=urlencode(data).encode('utf-8'))
        for key, value in self.cookies.items():
            req.cookies[key] = value

        resp = req.get_response(self.app)
        # naively handle cookies. This can be made more robust later
        cookie_string = resp.headers.get("set-cookie")
        if cookie_string:
            self._set_cookie(cookie_string)

        return resp



import unittest
from webob import Request
import pybald
from pybald import context
from pybald.core.router import Router
from pybald.core.controllers import Controller, action
from pybald.core.middleware.errors import ErrorMiddleware

import logging
log = logging.getLogger(__name__)

not_found_response = '404 Not Found\n\nThe resource could not be found.\n\n No URL match  '
stack_trace_head = '''<html>\n<head>\n    <title>Pybald Runtime Error</title>\n'''
general_fault_response = '500 Internal Server Error\n\nThe server has either erred or is incapable of performing the requested operation.\n\n General Fault  '


def map(urls):
    # errors
    urls.connect('throw_exception', r'/throw_exception', controller='sample',
                 action='throw_exception')


class SampleController(Controller):
    @action
    def index(self, req):
        self.sample_variable = "Testing"

    @action
    def throw_exception(self, req):
        raise Exception("This is a test exception")


test_conf = dict(database_engine_uri='sqlite:///:memory:',
                 env_name="SampleTestProjectEnvironment",
                 debug=True)


class TestErrors(unittest.TestCase):
    def setUp(self):
        context._reset()

    def tearDown(self):
        context._reset()

    def test_stack_trace(self):
        "When in debug mode, throw an Exception and generate a stack trace"
        pybald.configure(config_object=test_conf)
        app = Router(routes=map, controllers=[SampleController])
        app = ErrorMiddleware(app)

        try:
            resp = Request.blank('/throw_exception').get_response(app)
        except Exception as err:
            self.fail("Exception Generated or Fell Through Error Handler {0}".format(err))
        self.assertEqual(resp.status_code, 500)
        self.assertEqual(stack_trace_head, str(resp.text)[:len(stack_trace_head)])

    def test_non_stack_trace(self):
        "When *NOT* in debug mode, throw an Exception and return a generic error"
        test_w_debug_conf = test_conf.copy()
        test_w_debug_conf.update(dict(debug=False))
        pybald.configure(config_object=test_w_debug_conf)
        app = Router(routes=map, controllers=[SampleController])
        app = ErrorMiddleware(app)
        try:
            resp = Request.blank('/throw_exception').get_response(app)
        except Exception as err:
            self.fail("Exception Generated or Fell Through Error Handler {0}".format(err))
        self.assertEqual(resp.status_code, 500)
        self.assertEqual(str(resp.text), general_fault_response)

    def test_404(self):
        "Return a 404 response"
        pybald.configure(config_object=test_conf)
        app = Router(routes=map, controllers=[SampleController])
        app = ErrorMiddleware(app)
        try:
            resp = Request.blank('/not_there').get_response(app)
        except Exception as err:
            self.fail("Exception Generated or Fell Through Error Handler {0}".format(err))
        self.assertEqual(resp.status_code, 404)
        self.assertEqual(str(resp.text), not_found_response)


if __name__ == "__main__":
    unittest.main()



'''Package containing support code for pybald, static servers / etc'''


from .text import (camel_to_underscore,
                  underscore_to_camel,
                  ordinal_suffix,
                  pluralize,
                  strip_accents)



import argparse
from pybald.util.console import start_console
from pybald.util.dev_server import start_dev_server


def start(app):
    '''Read the command line arguments passed in and start either the
    console or the development server.

    usage: [-h] {console,serve}
    '''
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help='sub-command help',
                                       dest="subcommand")
    subparsers.required = True
    parser_console = subparsers.add_parser('console', help='console help')
    parser_console.set_defaults(run=start_console)
    parser_serve = subparsers.add_parser('serve', help='serve help')
    parser_serve.add_argument('--host', type=str)
    parser_serve.add_argument('--port', type=int)
    parser_serve.set_defaults(host='0.0.0.0', port=8080)
    parser_serve.set_defaults(run=start_dev_server)
    options = parser.parse_args()
    options.run(app, options)



import unittest
import pybald
from six.moves.urllib.parse import urlencode
from pybald.core.router import Router
from webob import Request, exc


class TestRouter(unittest.TestCase):
    def setUp(self):
        pybald.configure(config_object={'debug': False})
        def map(urls):
            urls.connect('test1', r'/test1', controller='test1')
            urls.connect('test2', r'/test2', controller='test2', action='index')
            urls.connect('method_test1', r'/method', controller='test2',
                         action='get', conditions=dict(method=["GET"]))
            urls.connect('method_test2', r'/method', controller='test2',
                         action='delete', conditions=dict(method=["DELETE"]))
            # test3 is a 404 missing url
            urls.connect('test4', r'/test4', controller='test4', action='index')
            urls.connect('test5', r'/test5', controller='test2',
                         action='_iminvalid')
            urls.redirect('/here', '/there', _redirect_code='302 Found')

        class Test1Controller(object):
            def index(self, environ, start_response):
                start_response('200 OK', [('Content-Type','text/plain')])
                return ["test1"]

        class Test2Controller(object):
            def index(self, environ, start_response):
                start_response('200 OK', [('Content-Type','text/plain')])
                return ["test2"]

            def delete(self, environ, start_response):
                start_response('200 OK', [('Content-Type','text/plain')])
                return ["test2_delete"]

            def get(self, environ, start_response):
                start_response('200 OK', [('Content-Type','text/plain')])
                return ["test2_get"]

            def _iminvalid(self, environ, start_response):
                start_response('200 OK', [('Content-Type','text/plain')])
                return ["test5_invalid_action"]

        self.app = Router(routes=map, controllers=[Test1Controller, Test2Controller])

    def test_controller_match(self):
        '''Request a URL, match a controller'''
        r = Request.blank('/test1')
        resp = r.get_response(self.app)
        assert resp.body == 'test1'

    def test_no_match(self):
        '''Request a url that doesn't match, throw HTTP Not Found exception'''
        r = Request.blank('/test3')
        try:
            r.get_response(self.app)
        except exc.HTTPNotFound:
            pass

    def test_invalid_action(self):
        '''Fail to match an invalid action that starts with an underscore'''
        r = Request.blank('/test5')
        try:
            r.get_response(self.app)
        except exc.HTTPNotFound as err:
            assert err.args[0] == "Invalid Action"

    def test_no_controller_in_registry(self):
        '''Match controller variable but miss on registry'''
        r = Request.blank('/test4')
        with self.assertRaises(exc.HTTPNotFound) as context:
            r.get_response(self.app)

    def test_method_override(self):
        '''Send a `delete` _method data POST, the method is overridden'''
        r = Request.blank('/method',
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body=urlencode({'_method': 'delete'}).encode('utf-8'))
        resp = r.get_response(self.app)
        assert r.method == "DELETE"
        assert resp.body == 'test2_delete'

    def test_not_method_override(self):
        '''Send a `delete` _method url arg as GET, the method is NOT overridden'''
        r = Request.blank('/method?_method=delete')
        resp = r.get_response(self.app)
        assert r.method != "DELETE"
        assert resp.body == 'test2_get'

    def test_redirect(self):
        '''Fetch '/here' and redirect to '/there' '''
        r = Request.blank('/here')
        resp = r.get_response(self.app)
        assert resp.status_code == 302
        assert resp.headers['location'] == 'http://localhost/there'



import sys
from sys import stdout


def start_dev_server(app, options=None):
    '''Create a development web WSGI server using the standard library
    wsgiref module and serve the pybald application.

    This is not intended for production use.

    :param app: wsgi application passed in
    :param options: an object or named tuple containing the options for the
                    web server such as host and port.
    '''
    import os
    from pybald.context import config
    from collections import namedtuple
    from wsgiref.simple_server import make_server
    from pybald.util.static_serve import StaticServer
    if not options:
        defaults = namedtuple('defaults', ['host', 'port'])
        options = defaults('0.0.0.0', 8080)
    # called directly runs a simple dev server
    # add the static server component
    my_app = StaticServer(app, path=os.path.join(config.path, 'public'))
    httpd = make_server(options.host, options.port, my_app)
    stdout.write("Serving on {0}:{1}...\n".format(options.host, options.port))
    httpd.serve_forever()
    sys.exit(1)



#!/usr/bin/env python
# encoding: utf-8
import unittest
from mako.template import Template
from pybald.util import text
import pybald
from pybald import context


class TestTextUtils(unittest.TestCase):
    def setUp(self):
        pybald.configure(config_file="tests/sample_project/project.py")
        # from tests.sample_project.sample import app

    def tearDown(self):
        context._reset()

    def test_pluralize(self):
        '''Pluralize some words'''
        test_data = [('fox', 'foxes'),
                     ('quiz', 'quizzes'),
                     ('beach', 'beaches'),
                     ('test', 'tests'),
                     ('party', 'parties')]
        for singular, expected_plural in test_data:
            self.assertEqual(text.pluralize(singular), expected_plural)

    def test_ordinal_suffixes(self):
        '''Get the ordinal suffix for some numbers'''
        test_data = [(1, 'st'),
                     (2, 'nd'),
                     (17, 'th'),
                     (101, 'st'),
                     (33, 'rd'),
                     (-52, 'nd')]
        for number, expected_suffix in test_data:
            self.assertEqual(text.ordinal_suffix(number), expected_suffix)

    def test_camel_to_underscore(self):
        '''Convert camelcase to undesrscore separated'''
        test_data = [('TestOne', 'test_one'),
                     ('test123BigBang', 'test123_big_bang'),
                     ('HTMLTestOne', 'html_test_one'),
                     ('Justacap', 'justacap'),
                     ('PybaldIsTHEGreatest', 'pybald_is_the_greatest')]
        for camel, expected_underscore in test_data:
            self.assertEqual(text.camel_to_underscore(camel), expected_underscore)

    def test_underscore_to_camel(self):
        '''Convert underscore separated to camel case'''
        test_data = [('TestOne', 'test_one'),
                     ('Test123BigBang', 'test123_big_bang'),
                     ('HtmlTestOne', 'html_test_one'),
                     ('Justacap', 'justacap'),
                     ('PybaldIsTheGreatest', 'pybald_is_the_greatest')]
        for expected_camel, underscore in test_data:
            self.assertEqual(text.underscore_to_camel(underscore), expected_camel)

    def test_accent_stripping(self):
        '''Strip diacriticals and accents from words'''
        test_data = [(u'¡hosta cabrón!', u'¡hosta cabron!'),
                     (u'mötley crüe', u'motley crue'),
                     (b'bytes', u'bytes'),
                     (u'français', u'francais'),
                     (u'resumé', u'resume')]

        for accented, unaccented in test_data:
            self.assertEqual(text.strip_accents(accented), unaccented)



import unittest
import pybald
from pybald import context


class TestOrm(unittest.TestCase):
    def setUp(self):
        context._reset()
        pybald.configure(config_object=dict(database_engine_uri='sqlite:///:memory:'))

    def tearDown(self):
        context._reset()

    def test_model_save(self):
        "Save a model"
        from pybald.db import models

        class SampleModel(models.Model):
            text = models.Column(models.Text)

        models.Model.metadata.create_all()
        test_model = SampleModel(text="This is just the first test")
        try:
            test_model.save().commit()
        except Exception as err:
            self.fail("Model save failed: {0}".format(err))

    def test_model_save_load(self):
        "Read back a model after saving it"
        from pybald.db import models

        class SampleModel(models.Model):
            text = models.Column(models.Text)

        models.Model.metadata.create_all()
        test_model = SampleModel(text="This is just a test")
        test_model.save().commit()
        test_model_read = SampleModel.get(id=1)
        assert test_model_read.text == "This is just a test"






import pybald
from pybald.core.router import Router
if __name__ == "__main__":
    from pybald.core.logs import default_debug_log
    default_debug_log()
context = pybald.configure(__name__)
from pybald.core.controllers import Controller, action
from pybald.core.middleware.errors import ErrorMiddleware
from pybald.db import models

def map(urls):
    urls.connect('home', r'/', controller='sample')
    urls.connect('variable_test', r'/variable_test/{variable}', controller='sample',
                 action='variable_test')
    # errors
    urls.connect('throw_exception', r'/throw_exception', controller='sample',
                 action='throw_exception')


class SampleModel(models.Model):
    text = models.Column(models.Text)


class SampleController(Controller):
    @action
    def index(self, req):
        self.sample_variable = "Testing"

    @action
    def throw_exception(self, req):
        raise Exception("This is a test exception")


app = Router(routes=map, controllers=context.controller_registry)
app = ErrorMiddleware(app)


if __name__ == "__main__":
    context.start(app)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import pybald
from pybald import context
import sqlalchemy.exc


class TestDbExt(unittest.TestCase):
    def setUp(self):
        context._reset()
        pybald.configure(config_object=dict(database_engine_uri='sqlite:///:memory:'))

    def tearDown(self):
        context._reset()

    def test_ascii_type_rejects_non_ascii(self):
        "ASCII type throws error on non-ascii data"
        from pybald.db import models

        class SampleModel(models.Model):
            text = models.Column(models.ASCII)

        models.Model.metadata.create_all()
        test_model = SampleModel(text=u"¡This is just the first test!")
        with self.assertRaises(sqlalchemy.exc.StatementError) as context:
            test_model.save().commit()

    def test_ascii_type(self):
        "ASCII type accepts unicode that can be encoded in ASCII"
        from pybald.db import models

        class SampleModel(models.Model):
            text = models.Column(models.ASCII)

        models.Model.metadata.create_all()
        test_model = SampleModel(text=u"This is just the first test!")
        test_model.save().commit()

    def test_save_json_type(self):
        "JSON type accepts dicts and stores JSON"
        from pybald.db import models

        class SampleModel(models.Model):
            data = models.Column(models.JSONEncodedDict)

        models.Model.metadata.create_all()
        test_model = SampleModel(data={"A": 1, "B": 2})
        test_model.save().commit()

    def test_read_json_type(self):
        "JSON type reads JSON from db and returns dict"
        from pybald.db import models

        class SampleModel(models.Model):
            data = models.Column(models.JSONEncodedDict)

        models.Model.metadata.create_all()
        test_model = SampleModel(data={"A": 1, "B": 2})
        test_model.save().commit()

        test_model_read = SampleModel.get(id=1)
        print(test_model_read.data)
        self.assertEqual(test_model_read.data, {"A": 1, "B": 2})

    def test_save_zip_pickle(self):
        from pybald.db import models

        class SampleModel(models.Model):
            data = models.Column(models.ZipPickleType)

        models.Model.metadata.create_all()
        test_model = SampleModel(data={"A": 1, "B": 2})
        test_model.save().commit()

    def test_read_zip_pickle(self):
        from pybald.db import models

        class SampleModel(models.Model):
            data = models.Column(models.ZipPickleType)

        models.Model.metadata.create_all()
        test_model = SampleModel(data={"A": 1, "B": 2})
        test_model.save().commit()

        test_model_read = SampleModel.get(id=1)
        self.assertEqual(test_model_read.data, {"A": 1, "B": 2})

    def test_mutation_dict(self):
        from pybald.db import models

        class SampleModel(models.Model):
            data = models.Column(models.MutationDict.as_mutable(models.JSONEncodedDict))

        models.Model.metadata.create_all()
        test_model = SampleModel(data={"A": 1, "B": 2})
        test_model.save().commit()
        # not changed
        self.assertNotIn(test_model, context.db.dirty)
        test_model.data["C"] = 3
        # now it *is* changed
        self.assertIn(test_model, context.db.dirty)
        test_model.commit()
        # changes saved
        self.assertNotIn(test_model, context.db.dirty)

        del test_model.data["A"]
        self.assertIn(test_model, context.db.dirty)
        test_model.commit()
        self.assertNotIn(test_model, context.db.dirty)

        test_model.data.update({"D": 4, "E": 5})
        self.assertIn(test_model, context.db.dirty)
        test_model.commit()
        self.assertNotIn(test_model, context.db.dirty)

        test_model.data.pop("B")
        self.assertIn(test_model, context.db.dirty)
        test_model.commit()
        self.assertNotIn(test_model, context.db.dirty)

        test_model.data.popitem()
        self.assertIn(test_model, context.db.dirty)
        test_model.commit()
        self.assertNotIn(test_model, context.db.dirty)


import sys
import os
import time
import signal

# ======
file_modification_times = {}


def is_modified(filename):
    '''
    Returns True if the file referenced by filename has been modified
    since the last time this funciton was called, otherwise False.
    '''
    new_modified_time = os.path.getmtime(filename)
    last_modified_time = file_modification_times.get(filename, None)
    if last_modified_time is None:
        file_modification_times[filename] = new_modified_time
        return False
    if last_modified_time < new_modified_time:
        file_modification_times[filename] = new_modified_time
        return True
    else:
        return False


def watch_module_files(pid=None):
    '''
    Continuously monitors all loaded modules and issues a POSIX SIGHUP if 
    any of the files associated with the loaded modules are changed during
    runtime.
    '''
    while True:
        changed = False
        for filename in filter(None, (getattr(module, '__file__', None) for
                                module in set(filter(None, sys.modules.values())))):
            changed = is_modified(filename)
            #print filename, changed
            if filename.endswith(".pyc") and os.path.exists(filename[:-1]):
                changed = changed or is_modified(filename[:-1])
                #print filename[:-1], changed
            if changed:
                print("File Changed!")
                os.kill(pid, signal.SIGHUP)
        time.sleep(1)



default_config = dict(
    # templates
    # =================
    template_default_helpers=[
        'from pybald.core.helpers import img, link, humanize, HTMLLiteral as literal, url_for',
        'from pybald.core import page'],
    # order important, html filter always first!
    template_default_filters=['h', 'unicode'],
    template_helpers=[],
    template_filesystem_check=True,
    template_path='app/views',
    cache_path='tmp/viewscache',
    page_options={},
    # database
    # =================
    global_table_args={},
    schema_reflection=False,
    database_engine_uri='',
    database_engine_args={},
    # Asset Pipeline
    # =================
    USE_CDN=False,
    DEFAULT_PROTOCOL="http",
    # Email
    # =================
    smtp_config={},
    # Misc
    # =================
    project_name=None,
    static_path="public",
    debug=True,
    email_errors=False,
    env_name="Default",
    host_name='localhost',
    path='',
    STATIC_SOURCES=None
)



#!/usr/bin/env python
# encoding: utf-8

#url_for takes these arguments as well
# anchor          specified the anchor name to be appened to the path
# host            overrides the default (current) host if provided
# protocol        overrides the default (current) protocol if provided
# qualified       creates the URL with the host/port information as
#                 needed

# TODO: add javascript escape code here so it's available in the template engine

from datetime import datetime
from routes import url_for
from mako import filters
from six.moves.urllib.parse import urlparse, ParseResult
import re
from routes import request_config
from pybald import context
import logging
log = logging.getLogger(__name__)

try:
    import pyhash
    hashfunc = pyhash.super_fast_hash()
except ImportError:
    log.warning("-"*80 + '''
    Warning
    -------
    Using python built-in hash() for asset URL generation. This is system
    implementation specific and may result in different hosts mapping static
    assets to different static hosts. That may cause inefficient use of browser
    caches. Optionally you can install pyhash to install additional fast,
    non-cryptographic, hashes that are not system dependent.

    pip install pyhash
'''+"-"*80)
    hashfunc = hash

try:
    type(unicode)
except NameError:
    unicode = str


# parse result keys
class AssetUrl(dict):
    '''
    Wraps urls and returns URL transformations when necessary.

    The primary use case is transforming the url for running static assets on a CDN.
    '''
    keys = ("scheme", "netloc", "path", "params", "query", "fragment")

    def __init__(self, url):
        self.raw_url = url
        super(AssetUrl, self).__init__(**dict(zip(self.keys, urlparse(url))))

    def __str__(self):
        return self.__html__()

    def __html__(self):
        '''Return a transformed URL if necessary (appending protocol and CDN)'''
        host = self.get('netloc', None)
        # Don't CDN urls with hosts we're not re-writing
        if host:
            if (context.config.STATIC_SOURCES is None or
                                           host not in context.config.STATIC_SOURCES):
                return self.raw_url
        if (context.config.USE_CDN and (context.config.CDN_HOST or context.config.STATIC_HOSTS)):
            # get the protocol for the current request
            # this requires the custom HTTP header X-Forwarded-Proto
            # set if running behind a proxy (or if SSL is terminated
            # upstream)
            try:
                protocol = request_config().protocol
            except AttributeError:
                # are we not in a request? Use a default protocol
                protocol = context.config.DEFAULT_PROTOCOL
            # use the round robin hosts to speed download when not https
            if protocol != "https" and context.config.STATIC_HOSTS:
                self['netloc'] = context.config.STATIC_HOSTS[hashfunc(self.raw_url) %
                                                      len(context.config.STATIC_HOSTS)]
            else:
                self['netloc'] = context.config.CDN_HOST
            # adjust the scheme of any link with a net location
            # to match the current request so we don't have mixed link
            # protocols
            if self['netloc']:
                self['scheme'] = protocol
        return ParseResult(**self).geturl()


class HTMLLiteral(unicode):
    '''This is an object to handle literal HTML in Mako template'''
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.__html__()

    def __html__(self):
        return self.value


class as_p(object):
    def __init__(self, input_str):
        self.str_value = input_str

    def __str__(self):
        return self.__html__()

    def __html__(self):
        '''Return the image in string form.'''
        lines = self.str_value.splitlines()
        return u"".join([u"<p>{0}</p>".format(line) for line in lines])


class tag(object):
    def set(self, **kargs):
        self.attribs.extend([u'''{0}="{1}"'''.format(k.rstrip('_'), v) for k, v in kargs.items()])
        return self


class img(tag):
    def __init__(self, src='', **kargs):
        self.img_src = src
        self.attribs = []
        self.set(**kargs)
        if "alt" not in kargs:
            self.set(alt=self.img_src)

    def __str__(self):
        return self.__html__()

    def __html__(self):
        '''Return the image in string form.'''
        return u'''<img src="{0}" {1} />'''.format(AssetUrl(str(self.img_src)),
                                                   " ".join(self.attribs))


class anchor(tag):
    def __init__(self, link_text='', name='', **kargs):
        self.link_text = filters.html_escape(link_text)
        if name:
            self.url = name
        else:
            self.url = self.link_text
        self.attribs = []
        self.set(**kargs)

    def __str__(self):
        return self.__html__()

    def __html__(self):
        '''Return the anchor in string form.'''
        # attr = " ".join(self.attribs)
        return u'''<a name="{0}" {1}>{2}</a>'''.format(self.url,
                                                       " ".join(self.attribs),
                                                       self.link_text)


class link(tag):
    def __init__(self, link_text='', **kargs):
        self.link_text = filters.html_escape(link_text)
        self.url = "#"
        self.attribs = []
        self.set(**kargs)

    def filter(self, filter_type="h"):
        self.link_text = filters.html_escape(self.link_text)
        return self

    def to(self, *pargs, **kargs):
        self.url = url_for(*pargs, **kargs)
        return self

    def to_raw_url(self, url):
        # Link directly with no url_for processing
        # (url_for can't deal with unicode!)
        self.url = url
        return self

    def __str__(self):
        return self.__html__()

    def __html__(self):
        '''Return the link in string form.'''
        attr = " ".join(self.attribs)
        return u'''<a href="{0}" {1}>{2}</a>'''.format(self.url, attr, self.link_text)


def plural(list_object):
    '''Return "s" for > 1 items'''
    if len(list_object) > 1:
        return "s"
    else:
        return ""


FRACTIONAL_SECOND = re.compile(r'\.\d+$')


def humanize(date_string, relative_date=None):
    '''Convert a date string into a 'humanized' relative string (like 1 day ago)'''
    date_format = "%Y-%m-%d %H:%M:%S"
    # strip decimal second precision
    date_string = FRACTIONAL_SECOND.sub('', date_string)
    try:
        date = datetime.strptime(date_string, date_format)
    except Exception:
        # if the date string format doesn't match, just return it
        return date_string
    if relative_date is None:
        now = datetime.now()
    else:
        now = relative_date
    delta = now - date
    plural = 's'
    if delta.days < 0:
        return "in the future"
    elif delta.days >= 30:
        date_format = "%d %b"
        if date.year != now.year:
            date_format += " '%y"
        return str(date.strftime(date_format).lstrip('0'))
    elif delta.days >= 14:
        weeks = delta.days / 7
        if weeks == 1:
            plural = ''
        return "%s week%s ago" % (str(int(weeks)), plural)
    elif delta.days >= 1:
        if delta.days == 1:
            plural = ''
        return "%s day%s ago" % (str(delta.days), plural)
    # > 1 hour, display in hours
    elif delta.seconds > 3600:
        hours = int(round(delta.seconds / 3600.0))
        if hours == 1:
            plural = ''
        return "%s hour%s ago" % (str(hours), plural)
    elif delta.seconds > 60:
        minutes = int(round(delta.seconds / 60.0))
        if minutes == 1:
            plural = ''
        return "%s minute%s ago" % (str(minutes), plural)
    else:
        return "just a moment ago"


# From django.utils.html: Javascript escape characters
_base_js_escapes = (
    ('\\', '\\u005C'),
    ('\'', '\\u0027'),
    ('"', '\\u0022'),
    ('>', '\\u003E'),
    ('<', '\\u003C'),
    ('&', '\\u0026'),
    ('=', '\\u003D'),
    ('-', '\\u002D'),
    (';', '\\u003B'),
    ('\u2028', '\\u2028'),
    ('\u2029', '\\u2029')
)

# From django.utils.html: Escape every ASCII character with a value less than 32.
_js_escapes = (_base_js_escapes +
              tuple([(chr(ascii_val), r"\u{:04X}".format(ascii_val)) for ascii_val in range(32)]))


def js_escape(value):
    """
    Hex encodes characters for use in JavaScript strings.
    (from django.utils.html)
    """
    if value:
        for bad, good in _js_escapes:
            value = value.replace(bad, good)
    return value



#!/usr/bin/env python
# encoding: utf-8

from wtforms import Form as WTForm
from wtforms import validators

from wtforms import (StringField, TextAreaField, SelectField,
                     BooleanField, DateField, DateTimeField, FloatField,
                     IntegerField, FileField, RadioField, SelectMultipleField,
                     SubmitField, HiddenField, PasswordField, Field,
                     DecimalField)

from pybald.context import render
from pybald.core.helpers import HTMLLiteral


class MockParams(dict):
    def getlist(self, key):
        return [self[key]]

    def __repr__(self):
        return type(self).__name__ + '(' + dict.__repr__(self) + ')'


class Form(WTForm):
    class Meta:
        def render_field(self, field, render_kw):
            return HTMLLiteral(field.widget(field, **render_kw))

    def render(self, *pargs, **kargs):
        return HTMLLiteral(render.form_render('fieldset', format='form',
                                              fieldset=self))

FieldSet = Form



#!/usr/bin/env python
# encoding: utf-8

import os
from pybald.context import config
from mako.template import Template
from mako.lookup import TemplateLookup
import re
import logging

log = logging.getLogger(__name__)

# templates follow name.FORMAT.template so this is a simple
# regex check of that pattern
TEMPLATE_PATTERN = re.compile(r'([^\.]+)\.([^\.]+)\.template$')

class TemplateEngine(object):
    '''
    The basic template engine, looks up templates and renders them.
    Uses the mako template system
    '''

    def __init__(self, template_path=None, cache_path=None, helpers=None):
        # base template helpers all pybald projects have
        self.template_helpers = [
            'from pybald.core.helpers import img,'
                                           ' link,'
                                           ' humanize,'
                                           ' HTMLLiteral as literal,'
                                           ' url_for',
            'from pybald.core import page']

        if config.template_helpers:
            self.template_helpers.extend(config.template_helpers)

        # set the default filters to auto-html escape all content
        self.default_filters = ['h', 'unicode']
        if config.template_default_filters:
            self.default_filters = config.template_default_filters

        self.project_path = config.path

        template_paths = []

        if config.template_path:
            project_template_path = os.path.join(self.project_path,
                                                 config.template_path)
            template_paths.append(project_template_path)

        # the pybald default templates, like forms and stack traces
        default_template_path = os.path.join(os.path.dirname(
                                            os.path.realpath(__file__)),
                                            'default_templates')
        template_paths.append(default_template_path)

        #caching compiled python template modules, None disables the cache
        if config.cache_path:
            project_cache_path = os.path.join(self.project_path,
                                              config.cache_path)
        else:
            project_cache_path = None

        fs_test = config.template_filesystem_check or config.debug or False
        self.template_args = dict(imports=self.template_helpers,
                                  input_encoding='utf-8',
                                  output_encoding='utf-8',
                                  default_filters=self.default_filters)
        self.lookup = TemplateLookup(directories=template_paths,
                                     module_directory=project_cache_path,
                                     filesystem_checks=fs_test,
                                     **self.template_args)

    def raw_template(self, template_text, data={}, *pargs, **kargs):
        '''Take a template as a string, compile it and use the default
        arguments to render the result and return the encoded string.

        :param template_text: The raw string form of a template, useful for
                              small snippets and testing
        :param data: the data to render
        :param kargs: Additional arguments to pass to the Template when
                      making the template object
        '''
        template_data = dict(list(config.page_options.items()) + list(data.items()))
        myargs = self.template_args.copy()
        myargs.update(kargs)
        mytemplate = Template(template_text, **myargs)
        return mytemplate.render(**template_data)

    def partial(self, template_name=None, format="html", **kargs):
        mytemplate = self._get_template(template_name, format=format)
        return mytemplate.render_unicode(**kargs)

    def form_render(self, template_name=None, format="form", **kargs):
        '''
        Render the form for a specific model using formalchemy.

        :param template_name: The name of the template to search for and
                              render for a form
        :param kargs: the data to render in the context of the form
                        template
        '''
        try:
            template_id = kargs['fieldset'].template_id
        except (KeyError, AttributeError):
            template_id = 'forms/{0}'.format(template_name)
        return self.partial(template_id, format, **kargs)

    def _get_template(self, template, format="html"):
        '''
        Retrieves the proper template from the Mako template system.

        :param template: The name of the template in the filesystem to retrieve
                         for rendering.
        :param format: A string specifying the format for the template (html,
                       json, xml, etc...), overrides the format specified in
                       the data dictionary.

        The _get_template method of the template engine constructs a template
        name based on the template_id and the format and retrieves it from the
        Mako template system.
        '''
        # format can't be None, set to html as a default
        if format is None:
            format = 'html'
        if TEMPLATE_PATTERN.search(template):
            template_file = template
        else:
            template_file = "/{0}.{1}.template".format(template.lower().lstrip('/'),
                                                   format.lower())
        log.debug("Using template: {0}".format(template_file))
        # TODO: Add memc caching of rendered templates
        # also need to check if the internal caching is good enough
        return self.lookup.get_template(template_file)

    def __call__(self, template=None, data={}, format="html"):
        '''
        Renders the template.

        :param template: The name of the template in the filesystem to retrieve
                         for rendering.
        :param data: A dictionary that represents the context to render inside
                     the template. Keys in this dictionary will be available
                     to the template.
        :param format: A string specifying the format type to return (e.g. 'html',
                       'json', 'xml')

        Calls _get_template to retrieve the template and then renders it.
        '''
        template_data = dict(list(config.page_options.items()) + list(data.items()))
        mytemplate = self._get_template(template, format)
        log.debug("Rendering template")
        return mytemplate.render(**template_data)




#!/usr/bin/env python
# encoding: utf-8
from webob import Request, Response


class SessionManager(object):
    '''Code to handle anonymous and user sessions, implemented as WSGI middleware.'''

    def __init__(self, application=None, days=14, session_class=None):
        self.session_class = session_class
        self.days = days
        if application:
            self.application = application
        else:
            # no pipeline so just generate a generic response
            self.application = Response()

    def __call__(self, environ, start_response):
        req = Request(environ)

        environ['pybald.session'] = self.session_class._before(req)

        # update or create the pybald.extension to populate controller instances
        environ.setdefault('pybald.extension', {})['session'] = environ['pybald.session']

        # call the next part of the pipeline
        resp = req.get_response(self.application)

        # execute any post-processing code for the session
        # this includes saving the session if necessary.
        environ['pybald.session']._after(req, resp)

        return resp(environ,start_response)




import unittest
import pybald
from pybald import context
from pybald.util.console import Console


test_config = dict(env_name="ConsoleTest",
                   config_object=True,
                   sample_config=True,
                   cache_path=None,
                   project_name="Console Testing",
                   debug=True,
                   path="",
                   database_engine_uri='')


class TestConsole(unittest.TestCase):
    def setUp(self):
        pybald.configure(config_object=test_config)

    def tearDown(self):
        context._reset()

    def test_create_console(self):
        '''Create a pybald console'''
        try:
            console = Console(project_name=pybald.context.config.project_name or pybald.context.name,
                      app=None, additional_symbols=[])
        except Exception as err:
            self.fail(err)



#!/usr/bin/env python
# encoding: utf-8
import logging
import logging.handlers
from textwrap import TextWrapper
log = logging.getLogger(__name__)


class WrappedFormatter(logging.Formatter):
    '''
    Formatter subclass that indents the messages by 20 characters and
    prepends sql> prompt.
    '''
    def __init__(self, *pargs, **kargs):
        logging.Formatter.__init__(self, *pargs, **kargs)
        self.sql_wrapper = TextWrapper(width=100,
                                       initial_indent=' ' * 15 + 'sql> ',
                                       subsequent_indent=' ' * 20)

    def format(self, record):
        fmt = logging.Formatter.format(self, record)
        wrapped_text = "{0}".format(self.sql_wrapper.fill(fmt))
        return wrapped_text

# sqlalchemy engine logger
engine_log = logging.getLogger('sqlalchemy.engine')
root_log = logging.getLogger()


def set_root_logger(level=logging.DEBUG, log_class=logging.StreamHandler):
    '''Adds a handler and log level to the root logger.

    By default, sets log level to DEBUG and the handler to a StreamHandler.
    '''
    # log all debug messages
    # pull the root logger and set it's logging to *level*
    # root = logging.getLogger()
    root_log.setLevel(level)

    if not root_log.handlers:
        handler = log_class()
        root_log.addHandler(handler)


def set_sql_logger(level=logging.INFO, log_class=logging.StreamHandler):
    '''Adds a handler and log level to the sqlalchemy engine logger.
    '''
    if engine_log.handlers:
        for handler in engine_log.handlers:
            engine_log.removeHandler(handler)
    # setup indented logging for SQL output
    sql_log_handler = log_class()
    formatter = WrappedFormatter("%(message)s")
    sql_log_handler.setFormatter(formatter)
    # For SQL log, INFO is better than DEBUG
    engine_log.setLevel(level)
    engine_log.addHandler(sql_log_handler)
    # avoid repeat log entries by stopping propagation
    # of the engine log
    # we're handling it with the indented logger above
    engine_log.propagate = False


# TODO: change thie default level to ERROR and always
# enable logging
def default_debug_log(level=logging.DEBUG, log_class=logging.StreamHandler):
    set_root_logger(level, log_class)
    # the the sql logger to INFO since DEBUG is to chatty
    set_sql_logger(logging.INFO, log_class)


def enable_sql_log():
    '''Function to turn on debug SQL output for SQLAlchemy'''
    engine_log.setLevel(logging.INFO)


def disable_sql_log():
    '''Function to turn off debug SQL output for SQLAlchemy'''
    engine_log.setLevel(logging.ERROR)


class LogPoint(object):
    '''
    WSGI middleware to output a log message before and after this point in
    the wsgi pipeline
    '''
    def __init__(self, application, begin_message="start", end_message="end",
                 width=79, fillchar='-'):
        self.begin_message = begin_message
        self.end_message = end_message
        self.application = application
        self.log = '{{0:{0}^{1}}}'.format(fillchar, width)

    def __call__(self, environ, start_response):
        log.debug(self.log.format(' {0} '.format(self.begin_message)))
        resp = self.application(environ, start_response)
        log.debug(self.log.format(' {0} '.format(self.end_message)))
        return resp





#!/usr/bin/env python
# encoding: utf-8
'''HTML page helper functions as well as simple asset tag handling.'''

import os
from pybald import context
from pybald.core.helpers import HTMLLiteral, AssetUrl
import logging
log = logging.getLogger(__name__)

# TODO - this singleton should be replaced
asset_tag_cache = {}

def compute_asset_tag(filename, pattern='{filename}{extension}?v={tag}'):
    '''
    Create a unique signature for a file.

    This asset tag is used for cache busting.
    '''
    asset_tag = asset_tag_cache.get(filename, None)
    try:
        if not asset_tag:
            asset_tag = str(
                int(round(os.path.getmtime(os.path.join(context.config.path,
                                                        "public",
                                                        filename.lstrip("/"))))
                ))
        asset_tag_cache[filename] = asset_tag
    except OSError:
        asset_tag_cache[filename] = "xxx"
    filename, ext = os.path.splitext(filename)
    return pattern.format(filename=filename, tag=asset_tag, extension=ext)


def add_js(filename):
    '''Add a script tag to a template.

    This helper function is also config aware and will re-write asset urls
    based on CDN and other rules.'''
    return HTMLLiteral('''<script src="{0}"></script>'''.format(
                                        AssetUrl(compute_asset_tag(filename))))


def add_css(filename, media="screen"):
    '''Add a link/css tag to a template.

    This helper function is also config aware and will re-write asset urls
    based on CDN and other rules.'''
    return HTMLLiteral('''<link type="text/css" href="{0}" media="{1}" rel="stylesheet">'''.format(
                                                AssetUrl(compute_asset_tag(filename)),
                                                str(media)))


def add_extern_css(filename, media="screen"):
    return HTMLLiteral(('''<link type="text/css" href="{0}"'''
            ''' media="{1}" rel="stylesheet">''').format(
                                                filename,
                                                str(media)))





import unittest
import pybald
from pybald import context
from pybald.core.helpers import AssetUrl
from routes import request_config


test_config = dict(STATIC_SOURCES=["example_cdn.com", "pybald.com"],
                   STATIC_HOSTS=["s0.sample.com"],
                   CDN_HOST="awesomecdn.com",
                   DEFAULT_PROTOCOL="https")

my_static_resource = "/static_stuff.js"
my_static_resource_explicit = "http://example_cdn.com/static_stuff.js"
not_owned_static_resource = "https://bing.com/static_stuff.js"


class TestStaticAssets(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        if hasattr(request_config(), "protocol"):
            del request_config().protocol
        context._reset()

    def test_dont_cdn_relative_url_rewrite_when_off(self):
        '''No re-writing if CDN is off'''
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = False
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource)
        self.assertEqual(str(asset_url), str(my_static_resource))

    def test_dont_cdn_absolute_url_rewrite_when_off(self):
        '''No re-writing if CDN is off'''
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = False
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource_explicit)
        self.assertEqual(str(asset_url), str(my_static_resource_explicit))

    def test_cdn_relative_url_rewrite_http(self):
        '''Test that relative link gets changed to a CDN link'''
        request_config().protocol = "http"
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = True
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource)
        self.assertEqual(str(asset_url), 'http://s0.sample.com/static_stuff.js')

    def test_cdn_absolute_url_rewrite_http(self):
        '''Test that an absolute link gets changed to a CDN link'''
        request_config().protocol = "http"
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = True
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource_explicit)
        self.assertEqual(str(asset_url), 'http://s0.sample.com/static_stuff.js')

    def test_cdn_url_rewrite_https(self):
        '''Test that the CDN rewrite is smart enough to use the raw CDN if
        https and we don't have https certs for static hosts.'''
        request_config().protocol = "https"
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = True
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource)
        self.assertEqual(str(asset_url), 'https://awesomecdn.com/static_stuff.js')

    def test_dont_cdn_url_rewrite_for_unowned_link(self):
        '''Don't rewrite links on hosts we don't own'''
        request_config().protocol = "https"
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = True
        pybald.configure(a_test_config)
        asset_url = AssetUrl(not_owned_static_resource)
        self.assertEqual(str(asset_url), str(not_owned_static_resource))

    def test_default_protocol(self):
        '''Test that the CDN rewrite is smart enough to use the raw CDN if
        https and we don't have https certs for static hosts.'''
        a_test_config = test_config.copy()
        a_test_config["USE_CDN"] = True
        pybald.configure(config_object=a_test_config)
        asset_url = AssetUrl(my_static_resource)
        self.assertEqual(str(asset_url), 'https://awesomecdn.com/static_stuff.js')


import unittest
import pybald
from pybald import context
from pybald.core.controllers import (Controller, action, csrf_protected,
                                     CSRFValidationFailure)
from webob import Request
from six.moves.urllib.parse import urlencode


class MockStash(dict):
    def __call__(self, *pargs, **kargs):
        if not kargs and len(pargs) == 1:
            return self[pargs[0]]
        if kargs:
            self.update(kargs)


class MockSession(object):
    def __init__(self):
        self.stash = MockStash()


class TempController(Controller):
    @action
    @csrf_protected
    def test1(self, req):
        return 'Some data'


class TestControllers(unittest.TestCase):
    def setUp(self):
        context = pybald.configure(config_object=dict(env_name="ControllerTest"))

    def tearDown(self):
        context._reset()

    def test_missing_csrf_rejection(self):
        "Reject post to csrf protected action when no CSRF was generated"
        controller = TempController()
        controller.session = MockSession()
        app = controller.test1
        # request.get_response(app)
        # issue a post
        request = Request.blank('/',
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body=urlencode({'__csrf_token__': 'SOMEOTHERTOKEN'}).encode('utf-8'))
        try:
            request.get_response(app)
        except CSRFValidationFailure:
            pass
        else:
            self.fail('Failed to throw validation exception')

    def test_no_csrf_rejection(self):
        "Reject post to csrf protected action w/out token"
        controller = TempController()
        controller.session = MockSession()
        # issue a simulated 'get' to populate the CSRF
        request = Request.blank('/')
        app = controller.test1
        request.get_response(app)
        # issue a post
        request = Request.blank('/',
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body='Nocsrf'.encode('utf-8'))
        try:
            request.get_response(app)
        except CSRFValidationFailure:
            pass
        else:
            self.fail('Failed to throw validation exception')

    def test_wrong_csrf_reject(self):
        "Reject post to csrf protected action w/wrong token"
        controller = TempController()
        controller.session = MockSession()
        # issue a simulated 'get' to populate the CSRF
        request = Request.blank('/')
        app = controller.test1
        request.get_response(app)
        # issue a post with the token
        token = controller.session.stash.get('csrf_token')
        request = Request.blank('/',
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body=urlencode({'__csrf_token__': 'SOMEOTHERTOKEN'}).encode('utf-8'))
        try:
            request.get_response(app)
        except CSRFValidationFailure:
            pass
        else:
            self.fail('Wrong token accepted')

    def test_csrf_accept(self):
        "Accept post to csrf protected action w/token"
        controller = TempController()
        controller.session = MockSession()
        # issue a simulated 'get' to populate the CSRF
        request = Request.blank('/')
        app = controller.test1
        request.get_response(app)
        # issue a post with the token
        token = controller.session.stash.get('csrf_token')
        request = Request.blank('/',
                         content_type="application/x-www-form-urlencoded",
                         method="POST",
                         body=urlencode({'__csrf_token__': token}).encode('utf-8'))
        try:
            request.get_response(app)
        except CSRFValidationFailure:
            self.fail('Correct token not accepted')
        else:
            pass







import sys
from six import reraise as raise_
from sqlalchemy.exc import SQLAlchemyError
from pybald import context
from webob import Response
import logging
log = logging.getLogger(__name__)

class EndPybaldMiddleware(object):
    '''Utilitiy middleware to force remove current session at the end of
    the request.'''
    def __init__(self, application):
        self.application = application

    def __call__(self, environ, start_response):
        try:
            return self.application(environ, start_response)
        finally:
            # always, always, ALWAYS close the session regardless
            context.db.remove()


class DbMiddleware(object):
    '''The database middleware provides three behaviors, committing
    transactions at the end of the request, rolling back database transactions
    if errors occur and forcibly closing the session
    at the end of the web request to avoid dangling connections.'''
    def __init__(self, application=None):
        if application:
            self.application = application
        else:
            # no pipeline so just generate a generic response
            self.application = Response()

    def __call__(self, environ, start_response):
        # pass through if no exceptions occur, commit sessions on complete
        try:
            resp = self.application(environ, start_response)
            # commit any outstanding sql
            context.db.commit()
        # on any SQLAlchemy Errors, rollback the transaction
        except SQLAlchemyError:
            log.exception("SQLAlchemy Error")
            # excpt, detail, tb = sys.exc_info()
            context.db.rollback()
            raise
        else:
            return resp
        finally:
            # always, always, ALWAYS close the session regardless
            context.db.remove()



#!/usr/bin/env python
# encoding: utf-8
import os
import mimetypes
from datetime import datetime, timedelta
import io
from functools import partial


class StaticServer(object):
    '''
    A *very* simple static asset server. Do not use in production, just
    for test or development use.

    StaticServer is a WSGI application that can be included in the WSGI
    pipeline to intercept static requests (and perform file system lookups).
    If the file lookup fails, the pipeline is allowed to continue to any
    dynamic components.
    '''
    def __init__(self, application, path, browser_caching=False):
        """
        :param application: is the remaining WSGI pipeline to connect to.
        :param path: is directory where static files are stored
        :param browser_caching: whether to send 30d expires future headers
        """
        self.path = path
        self.application = application
        self.browser_caching = browser_caching

    def send_file(self, file_path, size):
        '''
        Read a file from the filesystem and yield 64k chunks of the file
        as a generator / iterable.
        '''
        BLOCK_SIZE = 64 * 1024
        with io.open(file_path, mode="rb") as f:
            file_read = partial(f.read, BLOCK_SIZE)
            for block in iter(file_read, b''):
                yield block

    def __call__(self, environ, start_response):
        '''
        A simple static file server.

        Should not be used in production environments.
        '''
        file_path = os.path.join(self.path, environ['PATH_INFO'].lstrip('/'))
        if not os.path.isfile(file_path):
            return self.application(environ, start_response)

        mimetype, encoding = mimetypes.guess_type(file_path)

        size = os.path.getsize(file_path)
        headers = [
            ("Content-Type", mimetype),
            ("Content-Length", str(size)),
        ]
        if self.browser_caching:
            expires = datetime.utcnow() + timedelta(days=30)
            headers.extend([
            ("Cache-Control", "public, max-age={0}".format(60 * 60 * 24 * 30)),
            ("Expires", "{0}".format(expires.strftime("%a, %e %b %Y %H:%M:%S GMT")))
                ])

        start_response("200 OK", headers)
        return self.send_file(file_path, size)




import re
import unicodedata

try:
    type(unicode)
except NameError:
    def unicode(txt, errors=None):
        return txt.decode('utf-8')

# first pass, anything before a CAPLower gets separated. i.e. CAP_Lower
#  123Lower -> 123_Lower
first_pass = re.compile(r'(.)([A-Z][a-z]+)')
# second pass, anything lowerCAP gets split then lowercased lower_cap
second_pass = re.compile(r'([a-z0-9])([A-Z])')


# technically this is PascalCase (or StudlyCaps)
def camel_to_underscore(text):
    '''Convert CamelCase text into underscore_separated text.'''
    return second_pass.sub(r'\1_\2', first_pass.sub(r'\1_\2', text)).lower()


def underscore_to_camel(text):
    '''Convert underscore_separated text into CamelCase text.'''
    return ''.join([token.capitalize() for token in text.split(r'_')])


ordinal_suffixes = {1: 'st', 2: 'nd', 3: 'rd'}


def ordinal_suffix(num):
    '''Returns the ordinal suffix of an interger (1st, 2nd, 3rd).'''
    if 10 <= abs(num) % 100 <= 20:
        return 'th'
    return ordinal_suffixes.get(abs(num) % 10, 'th')


plural_patterns = [(re.compile(pattern), re.compile(search), replace
                                        ) for pattern, search, replace in (
                         ('[^aeiouz]z$', '$', 's'),
                         ('[aeiou]z$', '$', 'zes'),
                         ('[sx]$', '$', 'es'),
                         ('[^aeioudgkprt]h$', '$', 'es'),
                         ('[^aeiou]y$', 'y$', 'ies'),
                         ('$', '$', 's'))]


def _build_plural_rule(input_pattern):
    pattern, search, replace = input_pattern
    return lambda word: pattern.search(word) and search.sub(replace, word)


plural_rules = list(map(_build_plural_rule, plural_patterns))


def pluralize(text):
    '''
    Returns a pluralized from of the input text following simple naive
    english language pluralization rules.

    Smart enough to turn fox into foxes and quiz into quizzes. Does not catch
    all pluralization rules, however.
    '''
    for rule in plural_rules:
        result = rule(text)
        if result:
            return result


def strip_accents(text):
    '''
    Strip diacriticals from characters. This will change the meaning of words
    but for places where unicode can't be used (or ASCII only) francais looks
    better than fran-ais or fran?ais.
    '''
    if isinstance(text, bytes):
        return unicode(text, errors='ignore')
    return ''.join((c for c in unicodedata.normalize('NFD', text) if
                                              unicodedata.category(c) != 'Mn'))




import unittest
from webob import Request
import pybald
from pybald import context
from pybald.core.router import Router
from pybald.core.controllers import Controller, action
from pybald.core.middleware.errors import ErrorMiddleware

import logging
log = logging.getLogger(__name__)


def map(urls):
    # errors
    urls.connect('test_xss_url_param', r'/test_xss_url_param', controller='xss',
                 action='test_url_param')


class XssController(Controller):
    @action
    def test_url_param(self, req):
        template = '''
<div>${url_param}</div>
'''
        data = {'url_param': req.GET['url_parameter']}
        return context.render.raw_template(template, data)


test_conf = dict(database_engine_uri='sqlite:///:memory:',
                 env_name="SampleTestProjectEnvironment",
                 debug=True)


class TestXss(unittest.TestCase):
    def setUp(self):
        context._reset()

    def tearDown(self):
        context._reset()

    def test_url_param_injection(self):
        '''Test that rendering a url parameter escapes any dangerous html'''
        pybald.configure(config_object=test_conf)
        app = Router(routes=map, controllers=[XssController])
        app = ErrorMiddleware(app)

        resp = Request.blank('/test_xss_url_param?url_parameter=%3C/script%3E%3Cscript%3Ealert%28document.cookie%29%3C/script%3E').get_response(app)
        self.assertEqual(resp.body,
                        b'\n<div>&lt;/script&gt;&lt;script&gt;alert(document.cookie)&lt;/script&gt;</div>\n')


if __name__ == "__main__":
    unittest.main()



from threading import local
import logging
log = logging.getLogger(__name__)


class Proxy(object):
    '''A convenience proxy implementing most of the pass through methods.'''
    def __init__(self, proxied_object):
        self._proxied_object = proxied_object

    def _proxied(self):
        return self._proxied_object

    def __dir__(self):
        return sorted(dir(self.__class__) + list(self.__dict__) + dir(self._proxied()))

    def __getattr__(self, attr):
        return getattr(self._proxied(), attr)

    def __setattr__(self, attr, value):
        setattr(self._proxied(), attr, value)

    def __delattr__(self, attr):
        delattr(self._proxied(), attr)

    def __getitem__(self, key):
        return self._proxied()[key]

    def __setitem__(self, key, value):
        self._proxied()[key] = value

    def __delitem__(self, key):
        del self._proxied()[key]

    def __call__(self, *args, **kw):
        return self._proxied()(*args, **kw)

    def __repr__(self):
        return repr(self._proxied())

    def __iter__(self):
        return iter(self._proxied())

    def __len__(self):
        return len(self._proxied())

    def __contains__(self, key):
        return key in self._proxied()

    def __bool__(self):
        return bool(self._proxied())

    def __nonzero__(self):
        return bool(self._proxied())

    def __add__(self, obj):
        return self._proxied().__add__(obj)

    def __radd__(self, obj):
        return self._proxied().__radd__(obj)

    @property
    def __class__(self):
        return self._proxied().__class__

    @property
    def __file__(self):
        return self._proxied().__file__


class AppAttributeProxy(Proxy):
    def __init__(self, parent, attribute):
        self.parent = parent
        self.attribute = attribute

    def _proxied(self):
        return getattr(self.parent._proxied(), self.attribute)

    def __setattr__(self, attr, value):
        if attr in ('parent', 'attribute', '_proxied'):
            object.__setattr__(self, attr, value)
        else:
            super(AppAttributeProxy, self).__setattr__(attr, value)

    def __getattr__(self, attr):
        if attr in ('parent', 'attribute', '_proxied'):
            return object.__getattribute__(self, attr)
        else:
            return getattr(self._proxied(), attr)


class AppContext(Proxy):
    '''A convenience proxy that represents the current active configured
    application.

    Stores data in a simple thread local on a stacked list.'''

    def __init__(self, *pargs, **kargs):
        self.__dict__['___threadlocal__'] = local()
        self.__dict__['___threadlocal__'].stack = []
        self.__dict__['___stack__'] = self.__dict__['___threadlocal__'].stack
        self.__dict__['_default'] = None

    def _push(self, obj):
        '''Push an application context onto the stack. The first one pushed
        will be designated the default context for a reset'''
        if not self.__dict__['_default']:
            self.__dict__['_default'] = obj
        self.__dict__['___stack__'].append(obj)

    def _pop(self):
        '''Pop an application context off of the stack and return it'''
        return self.__dict__['___stack__'].pop()

    def _reset(self):
        '''Clear out any accumulated contexts.

        This method is dangerous and useful mainly for testing.
        '''
        self.__dict__['___stack__'] = [self.__dict__['_default']]
        return None

    def _depth(self):
        '''Return the current depth of the application context stack'''
        return len(self.__dict__['__stack__'])

    def _proxied(self):
        '''Return the actual object proxied by this app context'''
        try:
            return self.__dict__['___stack__'][-1]
        except IndexError:
            return None

    def __getattr__(self, attr):
        '''For all attributes of the wrapped object, return an AttributeProxy.

        This attribute proxy allows importing attributes and still maintaining
        the proxy behavior to look up the current context when accessing
        variables.

        This also allows late binding runtime configuration variables. This
        could be dangerous if a typo is introduced or other issue, the
        late code won't show an import error and instead will throw an
        attribute error when the late bound variable is accessed.
        '''
        return AppAttributeProxy(self, attr)




import unittest
import pybald
from pybald import context
from pybald.core.django.paginator import Paginator, PageNotAnInteger, EmptyPage


class TestPaginator(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        context._reset()

    def test_page_1(self):
        query = ["item"] * 400
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(1)
        self.assertEqual(page.number, 1)
        self.assertFalse(page.has_previous())
        self.assertTrue(page.has_next())
        self.assertEqual(page.next_page_number(), 2)

    def test_page_5(self):
        query = ["item"] * 400
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(5)
        self.assertEqual(page.number, 5)
        self.assertTrue(page.has_previous())
        self.assertTrue(page.has_next())
        self.assertEqual(page.previous_page_number(), 4)
        self.assertEqual(page.next_page_number(), 6)

    def test_page_10(self):
        query = ["item"] * 400
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(10)
        self.assertEqual(page.number, 10)
        self.assertTrue(page.has_previous())
        self.assertFalse(page.has_next())
        self.assertEqual(page.previous_page_number(), 9)

    def test_only_one_page(self):
        query = ["item"] * 30
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 1)
        page = p.page(1)
        self.assertFalse(page.has_other_pages())
        self.assertEqual(page.number, 1)
        self.assertFalse(page.has_previous())
        self.assertFalse(page.has_next())

    def test_bad_page(self):
        query = ["item"] * 30
        p = Paginator(query, 40)
        with self.assertRaises(PageNotAnInteger) as ctxt:
            # 10 pages
            self.assertEqual(p.num_pages, 1)
            page = p.page("A")
            # self.assertFalse(page.has_other_pages())
            # self.assertEqual(page.number, 1)
            # self.assertFalse(page.has_previous())
            # self.assertFalse(page.has_next())

    def test_negative_page(self):
        query = ["item"] * 30
        p = Paginator(query, 40)
        with self.assertRaises(EmptyPage) as ctxt:
            self.assertEqual(p.num_pages, 1)
            page = p.page(-10)

    def test_non_page(self):
        query = ["item"] * 30
        p = Paginator(query, 40)
        with self.assertRaises(EmptyPage) as ctxt:
            self.assertEqual(p.num_pages, 1)
            page = p.page(3)

    def test_page_start(self):
        query = ["item"] * 400
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(10)
        self.assertEqual(page.start_index(), 361)

    def test_page_end(self):
        query = ["item"] * 400
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(8)
        self.assertEqual(page.end_index(), 320)

    def test_page_end_orphans(self):
        query = ["item"] * 385
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 10)
        page = p.page(10)
        self.assertEqual(page.end_index(), 385)

    def test_page_start_no_items(self):
        query = []
        p = Paginator(query, 40)
        # 10 pages
        self.assertEqual(p.num_pages, 1)
        page = p.page(1)
        self.assertEqual(page.start_index(), 0)


#!/usr/bin/env python
# encoding: utf-8


class UserManager(object):
    '''Code to manage users, implemented as WSGI middleware. 

    This middleware will set REMOTE_USER as well as the pybald context object 
    within the WSGI environ. This allows any other WSGI or pybald aware code
    to inspect the environ for the current authenticated user.'''

    def __init__(self, application=None, user_class=None):
        self.user_class = user_class
        if application is None:
            raise ValueError("User Manager Middleware doesn't work stand alone, it is expected to wrap another application.")
        self.application = application

    def __call__(self, environ, start_response):
        session = environ.get('pybald.session', None)
        environ['REMOTE_USER'] = session.user

        if environ['REMOTE_USER']:
            # Continuously validate user sessions
            # TODO: Clean up our app-level login code a lot...

            # Look for a "valid" method on User
            try:
                valid = getattr(environ['REMOTE_USER'], "valid")
            except AttributeError:
                # (If this method isn't defined, do nothing.)
                pass
            else:
                # If available, call it, and expect a Boolean:
                # If False, end the session right now.
                if not valid():
                    environ['REMOTE_USER'] = None
                # Otherwise, do nothing

        # update or create the pybald.extension for other pybald aware WSGI code
        environ['pybald.extension'] = environ.get('pybald.extension', {})
        environ['pybald.extension']['user'] = environ['REMOTE_USER']
        # call the next part of the pipeline
        return self.application(environ, start_response)



#!/usr/bin/env python
# encoding: utf-8
# __init__.py
'''
Pybald
======

An MVC-like framework using many best-of-breed components (i.e. webob,
sqlalchemy).

(c) 2015 Michael Kowalchik
MIT License, see LICENSE file
'''
import imp
import sys
import os
from pybald.util.context import AppContext
import logging
from .default import default_config
from collections import namedtuple
log = logging.getLogger(__name__)

__version__ = '0.5.6'


def read_config_module(root_path='.', filename='project.py'):
    '''Read a configuration module and append it's path to sys.path'''
    # since the config module may have relative imports
    # add the _config files_ path to root before executing the config
    if root_path not in sys.path:
        sys.path.insert(1, root_path)
    filename = os.path.join(root_path, filename)
    config_module = imp.new_module("config")
    config_module.__dict__['__file__'] = filename
    config_module.__dict__['path'] = root_path
    with open(filename) as config_file:
        exec(compile(config_file.read(), filename, 'exec'),
             config_module.__dict__)
    return config_module


class Unconfigured(object):
    def __getattr__(self, key):
        raise RuntimeError("Pybald is not configured, you must run "
                           "pybald.configure() before context.{0} can"
                           " be used".format(key))


class DefaultApp(dict):
    def __init__(self, *pargs, **kargs):
        self.register('config', {})
        self.register('controller_registry', [])
        # self.register('model_registry', [])
        self.unconfigured = True
        super(DefaultApp, self).__init__(*pargs, **kargs)

    # def __getattr__(self, key):
    #     if key in ('models', 'engine', 'metdata'):
    #         raise RuntimeError("Pybald is not configured, you must run "
    #                            "pybald.configure() before context.{0} can"
    #                            " be used".format(key))
    #     else:
    #         return super(DefaultApp, self).__getattr__(key)

    def register(self, key, value):
        self[key] = value
        setattr(self, key, self[key])

# unconfigured application stack context
context = AppContext()
# register the context as a module
sys.modules['pybald.context'] = context
# push the default placeholder app onto the context stack
# the default allows for temporary storage of elements required
# for bootstrapping (and eliminates some sequencing requirements
# for configuration)
context._push(DefaultApp())

# the template engine and database session are built at config time
context_template = '''
from pybald.core.templates import TemplateEngine
from pybald.util.command_line import start
from pybald.core.models import ContextBoundModels
from pybald.db.db_engine import create_dump_engine

render = TemplateEngine()
dump_engine = create_dump_engine()
if config.database_engine_uri:
    models = ContextBoundModels()
    db = models.db
else:
    models = Unconfigured()
    db = Unconfigured()

def register(key, value):
    globals()[key] = value

'''


def create_config_dict_from_module(module):
    '''Converts a module into a dictionary filtering out common module
    level attributes'''
    return {key: getattr(module, key) for key in module.__dict__.keys() if
            not key.startswith('_') and key not in ('sys', 'os')}


def create_config_object(config_object):
    '''Convert a config dictionary to a config object named tuple
    and combines it with the default configuration dictionary.'''
    keys = set(default_config.keys()) | set(config_object.keys())
    ConfigObject = namedtuple("ConfigObject", keys)
    return ConfigObject(**dict(list(default_config.items()) +
                               list(config_object.items())))


def bootstrap(bootstrap_file=None):
    '''Execute the python file passed in. It's assumed that the bootstrap
    python module file will execute configure() before returning although
    not strictly necessary

    This is intended for complicated projects that need to do a lot of
    dynamic stuff before configuring the project. It's primarily a convenience
    hook that sets the sys.path to the current working directory before
    executing the module import.

    Use of this method should be rare, ask yourself if you really need it :)
    '''
    # since bootstrap can be anywhere, add bootstrap to sys.path as well
    if os.getcwd() not in sys.path:
        sys.path.insert(1, os.getcwd())
    possible_root_path = os.getcwd()
    root_path, filename = os.path.split(os.path.join(possible_root_path,
                                                     bootstrap_file))
    if root_path not in sys.path:
        sys.path.insert(1, root_path)
    filename = os.path.join(root_path, filename)
    bootstrap_module = imp.new_module("bootstrap")
    bootstrap_module.__dict__['__file__'] = filename
    with open(filename) as bootstrap_file:
        exec(compile(bootstrap_file.read(), filename, 'exec'),
             bootstrap_module.__dict__)
    return bootstrap_module


def configure(name=None, config_file=None, config_object=None, **kargs):
    '''
    Generate a dynamic context module that's pushed / popped on
    the application context stack.
    '''
    script_name = sys.argv[0]
    # always add current working directory to the path regardless
    # was trying to avoid this but too much path munging happening
    if os.getcwd() not in sys.path:
        sys.path.insert(1, os.getcwd())

    if config_object:
        if 'project_name' not in config_object:
            config_object['project_name'] = script_name
        # create a named tuple that's the combo of default plus input dict
        root_path = config_object['path'] = config_object.get('path',
                                                              os.getcwd())
        config = create_config_object(config_object)
        # add root path to config?
    elif config_file:
        # possible root path if relative filename specified
        possible_root_path = os.getcwd()
        root_path, filename = os.path.split(os.path.join(possible_root_path,
                                                         config_file))
        try:
            config_module = read_config_module(root_path=root_path,
                                               filename=filename)
        except IOError:
            log.exception("Config Error:\n"
                          "File Error or File not "
                          "found\n{0}".format(config_file))
            sys.exit(1)
        config = create_config_object(create_config_dict_from_module(config_module))
    elif kargs:
        if 'project_name' not in kargs:
            kargs['project_name'] = script_name

        root_path = kargs['path'] = kargs.get('path',
                                              os.getcwd())
        config = create_config_object(kargs)
    else:
        root_path = os.getcwd()
        log.warning("Warning: Using current path for the config "
                    "file {0}".format(root_path))
        if root_path not in sys.path:
            sys.path.insert(1, root_path)
        try:
            config_module = read_config_module(root_path=root_path,
                                               filename='project.py')
            log.warning("Found project.py in default path, "
                        "using for the config file")
        except IOError:
            config = create_config_object(default_config)
            log.warning("No config file, using default pybald configuration")
        else:
            config = create_config_object(create_config_dict_from_module(config_module))

    # if the discovered root_path is not in sys.path, add it in the least ugly
    # way possible
    if root_path not in sys.path:
        sys.path.insert(1, root_path)

    if config.debug:
        from pybald.core.logs import default_debug_log
        default_debug_log()

    new_context = imp.new_module("context")

    if context._proxied() and hasattr(context._proxied(), 'unconfigured'):
        # if we're at the root, consume any placeholder values
        placeholder = context._pop()
        new_context.__dict__.update(placeholder)
    else:
        new_context.__dict__['controller_registry'] = []
    # always set the runtime config
    new_context.__dict__['path'] = root_path
    new_context.__dict__['config'] = config
    new_context.__dict__['name'] = name
    new_context.__dict__['__file__'] = None
    new_context.__dict__['Unconfigured'] = Unconfigured
    # new_context.__dict__['__path__'] = None
    # now execute the app context with this config
    context._push(new_context)
    exec(compile(context_template, '<string>', 'exec'), new_context.__dict__)
    return new_context



#!/usr/bin/env python
# encoding: utf-8
import re
from webob import Request, Response, exc
from routes import Mapper, request_config, URLGenerator
# handle Mako's top level lookup
from mako import exceptions
from pybald.util import camel_to_underscore
import logging
log = logging.getLogger(__name__)


class Router(object):
    # class method match patterns
    controller_pattern = re.compile(r'(\w+)_controller')

    # add controllers=None to the call sig and use that for controller
    # loading
    def __init__(self, application=None, routes=None, controllers=None):
        '''
        Create a Router object, the core of the pybald framework.

        :param application:  WSGI application/middleware that is to be
                             *wrapped* by the router in the web app pipeline.

        :param routes: A routing function that takes a mapper (for parsing and
                       matching urls).

        :param controllers: A registry of all loaded controllers. This is
                            required to do routing lookups. This is also a
                            security precaution since only registered
                            controllers can be matched against.

        '''
        if routes is None or not callable(routes):
            raise TypeError("Route mapping is required. Please pass in a "
                            "routing function to the router as an arg to "
                            "Router init. "
                            "The routing function takes a routes mapper "
                            "object and uses it to contruct url mappings. "
                            "See pybald docs for more details.")

        self.controllers = {}
        # initialize Router
        # explicit turns off route memory and 'index' for
        # default action
        self.map = Mapper(explicit=False)
        routes(self.map)
        # debug print the whole URL map
        log.debug(str(self.map))
        self.load(controllers)

    def load(self, controllers):
        '''
        Walks the controller registry and builds the lookup table for
        controller classes to match against.

        Does some text munging to change the camel-case class names into
        underscore-separated url like names. (HomeController to home)

        :param controllers: A controller registry, a list of all controllers
                            that will be used with this application.

        All controller candidates are loaded into a hash to look up
        the matched "controller" urlvar against.

        The _controller suffix is removed from the module name for the url
        route mapping table (so controller="home" matches home_controller).

        This method is called only once at the start of a pybald application.
        '''

        controller_names = []
        controller_iterator = ((controller.__name__, controller) for
                                   controller in controllers)
        for name, controller in controller_iterator:
            controller_name = camel_to_underscore(name)
            try:
                controller_path_name = self.controller_pattern.search(
                                                    controller_name).group(1)
            except AttributeError:
                controller_path_name = controller_name
            controller_names.append(controller_path_name)
            # self.controllers holds paths to map to modules and controller
            # names
            self.controllers[controller_path_name] = controller

        # register the controller module names
        # with the mapper, creates the internal regular
        # expressions
        self.map.create_regs(controller_names)

    def __repr__(self):
        return "<Router Object>"

    def get_handler(self, urlvars):
        '''Method that returns the callable code mapped to this current
        request.

        This method can be overriden to change the behavior of mapping.
        '''
        controller_name, action_name = urlvars["controller"], urlvars["action"]
        #methods starting with underscore can't be used as actions
        if action_name.startswith("_"):
            raise exc.HTTPNotFound("Invalid Action")

        for key, value in urlvars.items():
            log.debug(u'''{0}: {1}'''.format(key, value))

        try:
            # create controller instance from controllers dictionary
            # using routes 'controller' returned from the match
            controller = self.controllers[controller_name]()
            handler = getattr(controller, action_name)
        # only catch the KeyError/AttributeError for the controller/action
        # search
        except (KeyError, AttributeError):
            raise exc.HTTPNotFound("Missing Controller or Action")

        return handler

    def __call__(self, environ, start_response):
        '''
        A Router instance is a WSGI app. It accepts the standard WSGI call
        signature of ``environ``, ``start_response``.

        The Router has a few jobs. First it uses the Routes package to
        compare the requested path to available url patterns that have
        been loaded and passed to the Router upon init.

        Router is the most *framework-like* component of Pybald. In addition to
        dispatching urls to controllers, it also allows 'method override'
        behavior allowing other HTTP methods to be invoked such as ``put`` and
        ``delete`` from web clients that don't support them natively.

        :param environ: WSGI CGI-like request environment

        :param start_response: WSGI callback for starting the response and
                               setting HTTP response headers

        '''
        req = Request(environ)
        req.errors = 'ignore'
        #method override
        #===============
        # for REST architecture, this allows a POST parameter of _method
        # to be used to override POST with alternate HTTP verbs (PUT, DELETE)
        if req.POST:
            override_method = req.POST.pop('_method', None)
            if override_method is not None:
                environ['REQUEST_METHOD'] = override_method.upper()
                log.debug("Changing request method to {0}".format(
                                                        environ["REQUEST_METHOD"]))

        results = self.map.routematch(environ=environ)
        if results:
            urlvars, route = results
        else:
            urlvars, route = {}, None

        url = URLGenerator(self.map, environ)
        config = request_config()

        # Your mapper object
        config.mapper = self.map
        # The dict from m.match for this URL request
        config.mapper_dict = urlvars
        config.host = req.host
        config.protocol = req.scheme
        # defines the redirect method. In this case it generates a
        # Webob Response object with the location and status headers
        # set
        config.redirect = lambda url: Response(location=url, status=302)

        # TODO: routing args is supposed to be pos, key dict
        environ.update({'wsgiorg.routing_args': ((url), urlvars),
                        'routes.route': route,
                        'routes.url': url,
                        'pybald.router': self})

        # Add pybald extension
        # the pybald.extension is a dictionary that can be used to copy state
        # into a running controller (usually handled by the @action decorator)
        environ.setdefault('pybald.extension', {})["url_for"] = url

        # debug print messages
        log.debug('{0:=^79}'.format(' {0} '.format(req.path_qs)))
        log.debug('Method: {0}'.format(req.method))

        # lifted from Routes middleware, handles 'redirect'
        # routes (map.redirect)
        if route and route.redirect:
            route_name = '_redirect_{0}'.format(id(route))
            location = url(route_name, **urlvars)
            return Response(location=location,
                            status=route.redirect_status
                            )(environ, start_response)

        if urlvars:
            handler = self.get_handler(urlvars)
        # No URL vars means nothing matched in the mapper function
        else:
            raise exc.HTTPNotFound("No URL match")

        try:
            # call the action we determined from the mapper
            return handler(environ, start_response)
        # This is a mako 'missing template' exception
        except exceptions.TopLevelLookupException:
            raise exc.HTTPNotFound("Missing Template")
        # All other program errors are allowed to bubble up
        # e.g. a 500 server error





sample_config = True
env_name = "SampleTestProjectEnvironment"
template_path = "tests/sample_project/"
cache_path = None
project_name = "Sample Project"
debug = True
path = ""
template_helpers = ['from pybald.core.helpers import as_p, anchor, plural, js_escape']
database_engine_uri = 'sqlite:///:memory:'




#!/usr/bin/env python
# encoding: utf-8
from six import with_metaclass
from functools import wraps
from webob import Request, Response, exc
import re
from pybald.util import camel_to_underscore
from routes import redirect_to
from pybald import context

import json
import random

import uuid
import logging
console = logging.getLogger(__name__)

controller_pattern = re.compile(r'(\w+)Controller')


class CSRFValidationFailure(exc.HTTPForbidden):
    pass


def csrf_protected(action_func):
    """
    Decorator to add CSRF (cross-site request forgery) protection to POST
    requests on an action. To use, include this decorator and provide the
    token in any submitted forms.

    For example, in the controller, do:

    @action
    @csrf_protected
    def my_action(self, req):
        ...

    And in the template:

    <form action='my_action' method='post'>
    ${csrf_input}
    ...
    </form>
    """
    CSRF_TOKEN_POST_VARIABLE = '__csrf_token__'

    @wraps(action_func)
    def replacement(self, req):
        if req.method == 'POST':
            try:
                if CSRF_TOKEN_POST_VARIABLE not in req.POST:
                    raise CSRFValidationFailure(
                        ("No %r value found in POST data. Please make sure "
                         "that a ${csrf_input} is used in the form template.")
                        % CSRF_TOKEN_POST_VARIABLE)

                if not self.session.stash.get("csrf_token"):
                    raise CSRFValidationFailure(
                        "CSRF validation failed: no validation token available "
                        "in this session.")

                provided_csrf_token = req.POST.get(CSRF_TOKEN_POST_VARIABLE)

                if provided_csrf_token != self.session.stash.get("csrf_token"):
                    raise CSRFValidationFailure(
                        "CSRF validation failed: token mismatch.")

                else:
                    # success! wipe out the used token
                    self.session.stash(csrf_token=None)
                    #del self.session.csrf_token

            except CSRFValidationFailure:
                # gentle mode, redirect to GET version of the page
                # return self._redirect_to(req.path_qs)
                raise

        # always stash a new token
        new_token = str(uuid.uuid4()).replace("-", "")
        self.session.stash(csrf_token=new_token)

        self.csrf_input = ("<input type='hidden' name='%s' value='%s' />" % (
            CSRF_TOKEN_POST_VARIABLE, new_token))

        return action_func(self, req)
    return replacement


# a no-op placeholder
def noop_func(*pargs, **kargs):
    '''Do nothing'''
    pass


def get_template_name(instance, method_name):
    '''
    Defines the template id to match against.

    :param instance: the instance to generate a template for
    :param method_name: the method to combine with the instance class

    template path = controller name + '/' + action name, except in the case
    of index. If the template is specified as part of the processed object
    return that, short circuiting any other template name processing

    This form may removed later, considered a candidate for deprecation
    '''
    template_id = getattr(instance, 'template_id', None)
    if template_id:
        return template_id
    # build a default template name if one isn't explicitly set
    try:
        template_root_name = camel_to_underscore(
                  controller_pattern.search(instance.__class__.__name__
                                            ).group(1))
    except AttributeError:
        template_root_name = ''
    return "/".join(filter(None, [template_root_name, method_name]))


# action / method decorator
def action(method):
    '''
    Decorates methods that are WSGI apps to turn them into pybald-style actions.

    :param method: A method to turn into a pybald-style action.

    This decorator is usually used to take the method of a controller instance
    and add some syntactic sugar around it to allow the method to use WebOb
    Request and Response objects. It will work with any method that
    implements the WSGI spec.

    It allows actions to work with WebOb request / response objects and handles
    default behaviors, such as displaying the view when nothing is returned,
    or setting up a plain text Response if a string is returned. It also
    assigns instance variables from the ``pybald.extension`` environ variables
    that can be set from other parts of the WSGI pipeline.

    This decorator is optional but recommended for making working
    with requests and responses easier.
    '''
    # the default template name is the controller class + method name
    # the method name is pulled during decoration and stored for use
    # in template lookups
    template_name = method.__name__
    # special case where 'call' or 'index' use the base class name
    # for the template otherwise use the base name
    if template_name in ('index', '__call__'):
        template_name = ''

    @wraps(method)
    def action_wrapper(self, environ, start_response):
        req = Request(environ)
        # add any url variables as members of the controller
        for varname, value in req.urlvars.items():
            # Set the controller object to contain the url variables
            # parsed from the dispatcher / router
            setattr(self, varname, value)

        # add the pybald extension dict to the controller
        # object
        for key, value in req.environ.setdefault('pybald.extension', {}).items():
            setattr(self, key, value)

        # TODO: fixme this is a hack
        setattr(self, 'request', req)
        setattr(self, 'request_url', req.url)

        # set pre/post/view to a no-op if they don't exist
        pre = getattr(self, '_pre', noop_func)
        post = getattr(self, '_post', noop_func)

        # set the template_id for this request
        self.template_id = get_template_name(self, template_name)

        # The response is either the controllers _pre code, whatever
        # is returned from the controller
        # or the view. So pre has precedence over
        # the return which has precedence over the view
        resp = (pre(req) or
                 method(self, req) or
                 context.render(template=self.template_id,
                             data=self.__dict__ or {}))
        # if the response is currently a string
        # wrap it in a response object
        if isinstance(resp, str) or isinstance(resp, bytes):
            resp = Response(body=resp, charset="utf-8")
        # run the controllers post code
        post(req, resp)
        return resp(environ, start_response)
    return action_wrapper


# def caching_pre(keys, method_name, prefix=''):
#     '''Decorator for pybald _pre to return cached responses if available.'''
#     if keys is None:
#         keys = []

#     def pre_wrapper(pre):
#         def replacement(self, req):
#             val = ":".join([prefix] + [str(getattr(self, k, '')) for
#                         k in keys] + [method_name])
#             self.cache_key = base64.urlsafe_b64encode(hashlib.md5(val).digest())
#             resp = project.mc.get(self.cache_key)
#             if resp:
#                 return resp
#             return pre(req)
#         return replacement
#     return pre_wrapper


# def caching_post(time=0):
#     '''Decorator for pybald _post to cache/store responses.'''
#     def post_wrapper(post):
#         def replacement(self, req, resp):
#             post(req, resp)
#             # only cache 2XX or 4XX responses
#             if (200 <= resp.status_code < 300) or (400 <= resp.status_code < 500):
#                 if 'X-Cache' not in resp.headers:
#                     resp.headerlist.append(('X-Cache', 'MISS'))
#                     project.mc.set(self.cache_key, resp, time)
#                 else:
#                     resp.headers['X-Cache'] = 'HIT'
#         return replacement
#     return post_wrapper

# regenerate a content_cache_prefix on every reload so that content will
# be force loaded after any full application restart
# This provides a way to cache static content for the duration of the
# application lifespan.
content_cache_prefix = hex(random.randrange(0, 2 ** 32 - 1))


# # memcache for actions
# def action_cached(prefix=content_cache_prefix, keys=None, time=0):
#     '''
#     Wrap actions and return pre-generated responses when appropriate.
#     '''
#     if keys is None:
#         keys = []

#     # def cached_wrapper(my_action_method):
#     #     @wraps(my_action_method)
#     #     def replacement(self, environ, start_response):
#     #         # bind newly wrapped methods to self
#     #         self._pre = caching_pre(keys,
#     #                                 my_action_method.__name__,
#     #                                 prefix=prefix)(self._pre
#     #                                     ).__get__(self, self.__class__)
#     #         self._post = caching_post(time)(self._post
#     #                                     ).__get__(self, self.__class__)
#     #         return my_action_method(self, environ, start_response)
#     #     # don't enable caching if requested
#     #     if project.DISABLE_STATIC_CONTENT_CACHE:
#     #         return my_action_method
#     #     return replacement
#     # return cached_wrapper

class RegistryMount(type):
    '''
    A registry creating metaclass that keeps track of all defined classes that
    inherit from a base class using this metaclass.
    '''
    # lifted almost verbatim from: http://martyalchin.com/2008/jan/10/simple-plugin-framework/
    def __init__(cls, name, bases, attrs):
        try:
            cls.registry.append(cls)
        except AttributeError:
            # this is processing the first class (the mount point)
            cls.registry = context.controller_registry

        return super(RegistryMount, cls).__init__(name, bases, attrs)


class Controller(with_metaclass(RegistryMount, object)):
    '''Base Controller that provides a registry mount

    The registry keeps track of all countrollers defined in the project. The
    Controller class also has some minor convenience methods attached.
    '''

    def __init__(self, *pargs, **kargs):
        for key, value in kargs.items():
            setattr(self, key, value)

    def _pre(self, req):
        pass

    def _post(self, req, resp):
        pass

    def _redirect_to(self, *pargs, **kargs):
        '''Redirect the controller'''
        return redirect_to(*pargs, **kargs)

    def _not_found(self, text=None):
        '''Raise the 404 http_client_error exception.'''
        raise exc.HTTPNotFound(text)

    def _status(self, code):
        '''Raise an http_client_error exception using a specific code'''
        raise exc.status_map[int(code)]

    def _JSON(self, data, status=200):
        '''Return JSON object with the proper-ish headers.'''
        res = Response(body=json.dumps(data),
                       status=status,
                       content_type="application/json",
                       charset='UTF-8')
        return res

# alias for backwards copatibility
BaseController = Controller



import unittest
import pybald
from pybald import context
from pybald.core.forms import validators
from pybald.core.forms import MockParams
expected_sample_fieldset = u'<div>\n<label for="text">Text</label>\n    <input id="text" name="text" type="text" value="">\n</div>\n'

test_config = dict(env_name="FormTest",
config_object=True,
sample_config=True,
# template_path="tests/sample_project/",
cache_path=None,
# project_name="Sample Project",
debug=True,
path="",
database_engine_uri='sqlite:///:memory:')


class TestForms(unittest.TestCase):
    def setUp(self):
        pybald.configure(config_object=test_config)

    def tearDown(self):
        context._reset()

    def test_create_fieldset(self):
        '''Create a form fieldset'''
        from pybald.core import forms
        from pybald.db import models

        class TestModelForForm(models.Model):
            text = models.Column(models.Text())

        class TestForm(forms.Form):
            text = forms.StringField()

        fs = TestForm()
        print(repr(fs.render().__html__()))
        self.assertEqual(fs.render().__html__(), expected_sample_fieldset)

    def test_bind_data_to_form_and_save(self):
        '''Create a new model from a form'''
        from pybald.core import forms
        from pybald.db import models

        class TestModelForForm(models.Model):
            text = models.Column(models.Text())

        class TestForm(forms.Form):
            text = forms.StringField()

        models.Model.metadata.create_all()
        fs = TestForm(data={'text': 'this is totally a test'})
        if fs.validate():
            tm = TestModelForForm()
            fs.populate_obj(tm)
            tm.save().commit()

    def test_edit_a_model(self):
        '''Edit an existing model by rebinding a form / fieldset'''
        from pybald.core import forms
        from pybald.db import models

        class TestModelForForm(models.Model):
            text = models.Column(models.Text())

        class TestForm(forms.Form):
            text = forms.StringField()

        models.Model.metadata.create_all()
        sm = TestModelForForm(text="This is just some sample text already in the model.")
        sm.save().commit()
        fs = TestForm(formdata=MockParams({'text': 'this is totally a test'}), obj=sm)
        def text_only(value, field):
            if not (isinstance(value, str)):
                raise validators.ValidationError('Value must be text')
        assert fs.text.data == 'this is totally a test'
        if fs.validate():
            fs.populate_obj(sm)
            sm.commit()
        saved_sample = TestModelForForm.get(id=1)
        assert saved_sample.text == 'this is totally a test'

    def test_validation(self):
        '''Fail to validate a form with a text value in an interger numeric validated field'''
        from pybald.core import forms
        from pybald.db import models

        def int_only(value, field):
            if not (isinstance(value, int)):
                raise validators.ValidationError('Value must be a number')

        class TestModelForForm(models.Model):
            text = models.Column(models.Text())

        class TestForm(forms.Form):
            text = forms.StringField('Text', [int_only])

        fs = TestForm(formdata=MockParams({'text': 'this is totally a test'}))
        if fs.validate():
            self.fail("The form validated when it shouldn't have")



#!/usr/bin/env python
# Copyright (c) 2015 Michael Kowalchik, TenZeroLab
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""Setup script for the pybald web framework."""

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages
import io
import os

import pybald
version = pybald.__version__

here = os.path.abspath(os.path.dirname(__file__))
with io.open(os.path.join(here, 'README.md'), encoding='utf8') as f:
    README = f.read()
try:
    with io.open(os.path.join(here, 'CHANGELOG.md'), encoding='utf8') as f:
        CHANGES = f.read() or ''
except IOError:
    CHANGES = ''

setup(name='pybald',
      version=version,
      description='A lightweight python web framework',
      long_description=README + '\n\n' + CHANGES + '\n\n',
      long_description_content_type='text/markdown',
      license='MIT',
      keywords='web framework',
      author='Michael Kowalchik',
      author_email='mikepk@tenzerolab.com',
      url='https://github.com/mikepk/pybald',
      project_urls={
        "Documentation": "http://pybald.com/",
        "Source Code": "https://github.com/mikepk/pybald",
      },
      packages=find_packages(),
      package_data={'pybald': ['core/default_templates/*.template', 'core/default_templates/forms/*.template']},
      install_requires=[
          "Routes==2.4.1", "SQLAlchemy==1.3.3",
          "WebOb==1.8.5", "Mako==1.0.7",
          "WTForms==2.2.1", "alembic==1.0.7", "six==1.12.0"
      ],
      extras_require={
        'docs': ['Sphinx>=1.6.2'],
        'tests': ['pytest>=3.1.1']
    },
)




import unittest
import pybald
from pybald import context

class TestConfig(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        context._reset()

    def test_default_config(self):
        "Start without a config, a default configuration is used"
        context = pybald.configure()
        assert context.config.debug == True
        assert context.config.env_name == 'Default'

    def test_read_from_file(self):
        "Read a pybald config from a file"
        context = pybald.configure(config_file="tests/sample_project/project.py")
        assert context.config.sample_config == True
        assert context.config.env_name == 'SampleTestProjectEnvironment'

    def test_read_from_object(self):
        "Pass pybald configuration as a dictionary of values"
        config_obj = dict(conf_object=True, env_name="TestFromObject")
        context = pybald.configure(config_object=config_obj)
        assert context.config.conf_object == True
        assert context.config.env_name == "TestFromObject"

    def test_missing_file(self):
        "Exit if specified config file is missing"
        with self.assertRaises(SystemExit) as context:
            pybald.configure(config_file="tests/sample_project/not_there.py")



from sqlalchemy.interfaces import ConnectionProxy

# A simple way to wrap database transactions in either a query timer.
# or more interestingly an instrumented stack tracer that annotates
# queries with the calling function. Allows the MySQL slow query log
# to be uesed to find what code is issuing slow queries.

class InstrumentedProxy(ConnectionProxy):
    '''A proxy to the underlying SQL execution engine.

    This proxy intercepts sql calls and allows timing and/or annotation
    of the SQL. With annotation, allows interesting patterns like using the
    database logging facilities to log slow queries and find what code
    is actually triggering the SQL by seeing the comment / annotation of
    the executed SQL statement.

    use by passing the proxy to the engine
    > engine = sa.create_engine(project.database_engine_uri, proxy=InstrumentedProxy(), **engine_args)
    '''
    def cursor_execute(self, execute, cursor, statement, parameters, context, executemany):
        '''The cursor execution proxy.

        This is the code that actually runs before running the underlying sql
        engine. In this case, attempts to walk up the current executing
        stack to find the code that triggered this query.
        '''
        try:
            import inspect
            stack = inspect.stack()
            # get the calling frame for the method/function calling
            # the sql command (taking into account the sql+execute functions)
            # max_stack = len(stack) - 1
            try:
                # for f in stack:
                #     frame = f[0]
                #     frame_info = inspect.getframeinfo(frame)
                #     debug = 'method:'+str(frame_info[2])+' file:'+str(frame_info[0])+' line:'+str(frame_info[1])
                #     print stack.index(f), debug

                frame = stack[14][0]
                frame_info = inspect.getframeinfo(frame)

                debug = 'method:'+str(frame_info[2])+' file:'+str(frame_info[0])+' line:'+str(frame_info[1])
                statement += ' -- '+debug
            except IndexError:
                pass
            return execute(cursor, statement, parameters, context)
        except:
            pass




import unittest
from mako.template import Template
import pybald
from pybald import context
from datetime import datetime, timedelta
from pybald.core.router import Router

now = datetime(2018, 6, 15, 3, 0)
this_year = now.year

helpers = (
('page_js', '''${page.add_js('/test.js')}''', b'''<script src="/test.js?v=None"></script>''', {}),
('page_css', '''${page.add_css('/sample.css')}''', b'''<link type="text/css" href="/sample.css?v=None" media="screen" rel="stylesheet">''', {}),
('date_humanize_moment_ago', '''${humanize(moment_ago, now)}''', b'just a moment ago', {'now': now, 'moment_ago': (now - timedelta(seconds=55)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_hour', '''${humanize(an_hour_ago, now)}''', b'1 hour ago', {'now': now, 'an_hour_ago': (now - timedelta(minutes=65)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_minutes', '''${humanize(minutes_ago, now)}''', b'15 minutes ago', {'now': now, 'minutes_ago': (now - timedelta(minutes=15)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_one_minute', '''${humanize(minute_ago, now)}''', b'1 minute ago', {'now': now, 'minute_ago': (now - timedelta(seconds=68)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_day', '''${humanize(a_day_ago, now)}''', b'1 day ago', {'now': now, 'a_day_ago': (now - timedelta(days=1)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_future', '''${humanize(in_the_future, now)}''', b'in the future', {'now': now, 'in_the_future': (now + timedelta(days=1)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_multi_weeks', '''${humanize(multi_weeks_ago, now)}''', b'2 weeks ago', {'now': now, 'multi_weeks_ago': (now - timedelta(days=14)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_more_than_30_days', '''${humanize(months_ago, now)}''',
     bytes(datetime.strftime(now - timedelta(days=45), "%d %b").lstrip('0').encode('utf-8')),
     {'now': now, 'months_ago': (now - timedelta(days=45)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_more_than_1_year', '''${humanize(years_ago, now)}''',
     bytes(datetime.strftime(now - timedelta(days=390), "%d %b &#39;%y").lstrip('0').encode('utf-8')),
     {'now': now, 'years_ago': (now - timedelta(days=390)).strftime("%Y-%m-%d %H:%M:%S")}),
# bytes(datetime.strftime(datetime(day=19, month=4, year=this_year), "%d %b &#39;%y"), 'utf-8'),
#     {'years_ago': (datetime.strptime('{this_year}-05-14 15:00:27'.format(this_year=this_year), "%Y-%m-%d %H:%M:%S") - timedelta(days=390)).strftime("%Y-%m-%d %H:%M:%S")}),
('date_humanize_bad_input', '''${humanize(bad_format)}''', b'Jan 1st 2045', {'bad_format': 'Jan 1st 2045'}),
('generate_link', '''${link('A test link').to('home')}''', b'<a href="/" >A test link</a>', {}),
('generate_raw_link', '''${link('A test link').to_raw_url('http://pybald.com')}''', b'<a href="http://pybald.com" >A test link</a>', {}),
('create_image', '''${img('/sample.jpg')}''', b'<img src="/sample.jpg" alt="/sample.jpg" />', {}),
('html_literal', '''${literal('<h2>This is a literal html h2</h2>')}''', b'<h2>This is a literal html h2</h2>', {}),
('html_escaping', '''${'<h2>This h2 tag should be escaped</h2>'}''', b'&lt;h2&gt;This h2 tag should be escaped&lt;/h2&gt;', {}),
('plural_more', '''dog${plural(['spot', 'fido'])}''', b'dogs', {}),
('plural_single', '''dog${plural(['spot'])}''', b'dog', {}),
('as_p', '''${as_p(text_with_line_breaks)}''', b'<p>This is just</p><p>A test</p><p>of paragraph breaks</p>', {"text_with_line_breaks": "This is just\nA test\nof paragraph breaks"}),
('anchor_no_name', '''${anchor('a block')}''', b'<a name="a block" >a block</a>', {}),
('anchor_name', '''${anchor('a block', 'heading1')}''', b'<a name="heading1" >a block</a>', {}),

('js_escape_hyphen', '''${js_escape('---- whee ----')}''', b'\\u002D\\u002D\\u002D\\u002D whee \\u002D\\u002D\\u002D\\u002D', {}),
('js_escape_single_quote', '''${js_escape(input)}''', r'whee \u0027 zing'.encode('utf8'), {'input': 'whee \' zing'}),
('js_escape_slash', '''${js_escape(input)}''', r'whee \u005C zork'.encode('utf8'), {'input': 'whee \\ zork'}),
('js_escape_double_quote', '''${js_escape(input)}''', r'whee \u0022'.encode('utf8'), {'input': 'whee "'}),
('js_escape_right_angle', '''${js_escape(input)}''', r'whee \u003E'.encode('utf8'), {'input': 'whee >'}),
('js_escape_left_angle', '''${js_escape(input)}''', r'whee \u003C'.encode('utf8'), {'input': 'whee <'}),
('js_escape_ampersand', '''${js_escape(input)}''', r'whee \u0026'.encode('utf8'), {'input': 'whee &'}),
('js_escape_equals', '''${js_escape(input)}''', r'whee \u003D'.encode('utf8'), {'input': 'whee ='}),
('js_escape_semicolon', '''${js_escape(input)}''', r'whee \u003B'.encode('utf8'), {'input': 'whee ;'}),
('js_escape_ascii_1', '''${js_escape(input)}''', b'whee \\u0001', {'input': 'whee \x01'}),
('js_escape_ascii_12', '''${js_escape(input)}''', b'whee \\u000C', {'input': 'whee \x0c'}),
('js_escape_ascii_24', '''${js_escape(input)}''', b'whee \\u0018', {'input': 'whee \x18'}),
('js_escape_ascii_31', '''${js_escape(input)}''', b'whee \\u001F', {'input': 'whee \x1f'}),
)


class TestTemplate(unittest.TestCase):
    def setUp(self):
        context = pybald.configure(config_file="tests/sample_project/project.py")

        def map(urls):
            urls.connect("home", r"/")

        self.app = Router(routes=map, controllers=[])

    def tearDown(self):
        context._reset()

    def test_template_lookup(self):
        '''Look up a template in the filesystem'''
        template = context.render._get_template('sample')
        # we've got a compiled template
        assert isinstance(template, Template)

    def test_template_render_with_data(self):
        '''Render a template with data.'''
        template = context.render._get_template('sample')
        assert template.render(sample_variable='sample') == b"<h1>Hello sample!</h1>"


def create_case(template, expected, data):
    def run_helper(self):
        result = context.render.raw_template(template, data)
        self.assertEqual(expected, result)
    return run_helper


def make_cases():
    for name, template, expected, data in helpers:
        test_method = create_case(template, expected, data)
        test_method.__name__ = 'test_template_helper_{}'.format(name)
        setattr(TestTemplate, test_method.__name__, test_method)

make_cases()



# # lifted from zzzeek's post on "Magic ORM"
# # this only works for SA 7, due to the
# # use of the event system.
# # Source: http://techspot.zzzeek.org/2011/05/17/magic-a-new-orm/
# # Copyright (I assume) Michael Bayer
# # =====================================
# from sqlalchemy.orm import (
#             class_mapper, mapper, relationship,
#             scoped_session, sessionmaker, configure_mappers
#         )
# from sqlalchemy.ext.declarative import declared_attr, declarative_base
# from sqlalchemy import event
# import re
#
# @event.listens_for(mapper, "mapper_configured")
# def _setup_deferred_properties(mapper, class_):
#     """Listen for finished mappers and apply DeferredProp
#     configurations."""
#
#     for key, value in class_.__dict__.items():
#         if isinstance(value, DeferredProp):
#             value._config(class_, key)
#
# class DeferredProp(object):
#     """A class attribute that generates a mapped attribute
#     after mappers are configured."""
#
#     def _setup_reverse(self, key, rel, target_cls):
#         """Setup bidirectional behavior between two relationships."""
#
#         reverse = self.kw.get('reverse')
#         if reverse:
#             reverse_attr = getattr(target_cls, reverse)
#             if not isinstance(reverse_attr, DeferredProp):
#                 reverse_attr.property._add_reverse_property(key)
#                 rel._add_reverse_property(reverse)
#
# class FKRelationship(DeferredProp):
#     """Generates a one to many or many to one relationship."""
#
#     def __init__(self, target, fk_col, **kw):
#         self.target = target
#         self.fk_col = fk_col
#         self.kw = kw
#
#     def _config(self, cls, key):
#         """Create a Column with ForeignKey as well as a relationship()."""
#
#         target_cls = cls._decl_class_registry[self.target]
#
#         pk_target, fk_target = self._get_pk_fk(cls, target_cls)
#         pk_table = pk_target.__table__
#         pk_col = list(pk_table.primary_key)[0]
#
#         if hasattr(fk_target, self.fk_col):
#             fk_col = getattr(fk_target, self.fk_col)
#         else:
#             fk_col = Column(self.fk_col, pk_col.type, ForeignKey(pk_col))
#             setattr(fk_target, self.fk_col, fk_col)
#
#         rel = relationship(target_cls,
#                 primaryjoin=fk_col==pk_col,
#                 collection_class=self.kw.get('collection_class', set)
#             )
#         setattr(cls, key, rel)
#         self._setup_reverse(key, rel, target_cls)
#
# class one_to_many(FKRelationship):
#     """Generates a one to many relationship."""
#
#     def _get_pk_fk(self, cls, target_cls):
#         return cls, target_cls
#
# class many_to_one(FKRelationship):
#     """Generates a many to one relationship."""
#
#     def _get_pk_fk(self, cls, target_cls):
#         return target_cls, cls
#
# class many_to_many(DeferredProp):
#     """Generates a many to many relationship."""
#
#     def __init__(self, target, tablename, local, remote, **kw):
#         self.target = target
#         self.tablename = tablename
#         self.local = local
#         self.remote = remote
#         self.kw = kw
#
#     def _config(self, cls, key):
#         """Create an association table between parent/target
#         as well as a relationship()."""
#
#         target_cls = cls._decl_class_registry[self.target]
#         local_pk = list(cls.__table__.primary_key)[0]
#         target_pk = list(target_cls.__table__.primary_key)[0]
#
#         t = Table(
#                 self.tablename,
#                 cls.metadata,
#                 Column(self.local, ForeignKey(local_pk), primary_key=True),
#                 Column(self.remote, ForeignKey(target_pk), primary_key=True),
#                 keep_existing=True
#             )
#         rel = relationship(target_cls,
#                 secondary=t,
#                 collection_class=self.kw.get('collection_class', set)
#             )
#         setattr(cls, key, rel)
#         self._setup_reverse(key, rel, target_cls)
#
# # =====================================
# # end of zzzeek's code for "Magic" ORM
# # =====================================


from sqlalchemy.orm.attributes import instance_state

import logging
log = logging.getLogger(__name__)


def child_relationships(obj):
    '''Get instances with child relationships to this obj.'''
    for related_obj, mapper, state, data in obj.__mapper__.cascade_iterator("save-update",
                                                                         instance_state(obj)):
        yield related_obj


# Caching / invalidation event handlers and functions.
def cached_objects(iter):
    '''Return objects from a session/iterator that are tagged as cached.'''
    for obj in iter:
        # check if the sqlalchemy object is tagged for caching
        if hasattr(obj, '__memcached__'):
            yield obj


def cached_objects_with_updates(iter):
    '''Return objects from a session/iterator that are tagged for caching and
    that need an update.'''
    for obj in iter:
        if getattr(obj, "__needs_update__", False):
            yield obj


class Cache(object):
    def __init__(self, memcached, session):
        self.session = session
        self.mc = memcached

    def memc_cache(self, func):
        '''Decorates a sqlalchemy method that returns entities to check
        the the cache first for the load arguments. Otherwise runs the
        method normally and marks the returned object as needing update'''
        def replacement(*pargs, **kargs):
            key = str("|".join(["{0}:{1}".format(k, v) for (k, v) in
                                                           sorted(kargs.items())]))
            cached_object = self.mc.get(key)
            if cached_object:
                # has the object been persisted at some point? then merge
                # with SA session
                if cached_object.is_persisted():
                    cached_object = self.session.merge(cached_object, load=False)
                cached_object.__needs_update__ = False
            else:
                # fetch the object the 'normal' way
                cached_object = func(*pargs, **kargs)
                # since we got this from the database, mark it with a
                # needs_update
                cached_object.__needs_update__ = True
            # mark cached object for cache-tracking
            cached_object.__memcached__ = True
            # reconstruct the relationship list, anything that's in the current
            # relationship map add it back to the object so we know what it came
            # out of the cache with.
            cached_object.__related__ = set([child_object for child_object in
                                            child_relationships(cached_object)])
            return cached_object
        return replacement

    def check_needs_update(self, session, *pargs, **kargs):
        '''Check for objects in the SQLAlchemy session that are tagged as cached
        and are modified and then tag them for cache update on post commit.'''
        # log.debug("CHECKING FOR CACHED OBJECTS...")
        for obj in cached_objects(session):
            # log.debug("FOUND OBJECT IN SESSION MARKED AS CACHED, OBJECT: {0}".format(type(obj)))
            if obj.__needs_update__:
                continue
            related = set([cr for cr in child_relationships(obj)])
            new_related_objects = (related - obj.__related__)
            if new_related_objects:
                # log.debug("NEW CHILD OBJECTS LOADED IN SESSION: {0}".format(new_related_objects))
                obj.__needs_update__ = True
                continue
            changed = set(self.session.dirty | self.session.deleted)
            if not new_related_objects and not changed:
                continue
            if obj in changed:
                # log.debug("OBJECT IS DIRTY MARKING FOR UPDATE, OBJECT: {0}".format(type(obj)))
                obj.__needs_update__ = True
                continue
            if (changed & related):
                # log.debug("THERE ARE DIRTY CHILDREN: {0}".format(dirty_children))
                obj.__needs_update__ = True
                continue

    def update_cached_objects(self, session):
        '''Write out (or invalidate) cache entries for modified entitites'''
        for obj in cached_objects_with_updates(session):
            # record needs update to false before inserting into the cache
            obj.__needs_update__ = False
            log.debug("UPDATING MEMCACHE")
            self.mc.set(obj.cache_key, obj)




import sys
import re
import sqlalchemy as sa
from pybald import context

def create_dump_engine():
    '''Create a sql engine that just prints SQL using the current dialect to
    STDOUT instead of executing it.

    This is useful especially to see what the DDL statements are for various dialects
    '''
    def dump(sql, *multiparams, **params):
        sys.stdout.write( str(sql.compile(dialect=dump_engine.dialect) ).strip() +"\n")

    def dialect():
        match = re.search(r'^([^\:]*\:\/\/)', context.config.database_engine_uri)
        if match:
            return match.group(1)
        else:
            return "sqlite://"
    dump_engine = sa.create_engine(dialect(), strategy='mock', executor=dump)
    return dump_engine